#!/usr/bin/perl

# Cobalt Backup/Restore v3
# Copyright 1997-1999 Cobalt Networks, Inc.
# Duncan Laurie [duncan@cobaltnet.com]
# backup/restore, supporting httpd, nfs, smb, ftp
#
# there's now a bunch of stuff to backup both the symlink and the
# actual directory/file to deal with storage addition.

use Cobalt::Product;
use Getopt::Long;
use Cobalt::Language;
use Cobalt::Locale;
use Cobalt::Vsite;
use Cobalt::User;
use Cobalt::Util;
use Cobalt::Meta;
use Net::FTP;
# POSIX used only for the setuid function.
use POSIX;
use CGI;
#use strict;

# 
# Main
# # # # #

# For security we set our effective uid to the uid of the remote user.
# Unless of course this is being run by admin or a scheduled backup.
# jeez.. Enough subscripts in that for you ?

my $me = $0;
my $DEBUG = 1;
my $header;
my $tarcmd = $Tar.' czpP --no-recursion --files-from';
my $temp = $Big_tmp;
my $web = 0;
my $version = 3;
my $filename;
my $ext = ".raq";
my $crondaily = $Cron_basedir.'daily';
my $cronweekly = $Cron_basedir.'weekly';
my $cronmonthly = $Cron_basedir.'monthly';
my $mountpoint = "/tmp/backup_nfs_mount.$$";
my $filelist = "/tmp/backup.$$.filelist";
my (@Gusers);
my (%args,%files,%smbinfo,%ftpinfo);
my @index;
my $remote_user = $ENV{ "REMOTE_USER" };

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my %backup_sets = ('complete',  "/etc /home $Vol_rootdir $Vaca_dir $AdminPages_dir $Majordomo_dir $Mailspool ",
		   'usergroup', "$Htusers_dir $Htgroup_dir $Mailspool ",
		   'user',      '',
		   'group',     '',
		   'site',      '',
		   'config',    "/etc $AdminPages_dir $Majordomo_dir /home/pgsql",
		   'mail',      "$Mailspool " );
my %exemptFiles = ("/etc/build", 1,
		   "/etc/initrunlvl", 1,
		   "$If_cfg_dir/ifcfg-eth0", 1,
		   "$If_cfg_dir/ifcfg-eth1", 1,
		   "$Net_cfg", 1,
		   "/home/log/httpd/adm_access", 1,
		   "/home/log/httpd/adm_error", 1,
		   "/home/log/httpd/access", 1,
		   "/home/log/httpd/error", 1,
		   "/home/quota.user", 1,
		   "/home/quota.group", 1);

if ( $ENV{'REMOTE_ADDR'} ) {
    %args = &parseinfo_web( @ARGV );
    $web = 1;
    &check_perms();
    &check_options();
} else {
    %args = &parseinfo_cmdline( @ARGV );
    &check_perms();
    &check_options();
}

# # # # # # # # # # #
#                   #
# S C H E D U L E D #
#                   #
# # # # # # # # # # #
if (defined($args{'scheduled'}) && ($args{'scheduled'} ne "420"))
{
    my ($s,$cronfile,$i,$schedule,$scheduleheader,@cmd,@crons);
    my ($existed,$sch,$uname);

    my $sched = $args{'scheduled'};
    my $mygr = $args{'mygroup'};

    if (defined($remote_user)) {
	$uname = $remote_user;
    } else {
	$uname = (getpwuid($<))[0];
    }

    if (defined($mygr)) {
	$sch = "/scheduled-backup."."$mygr";

	my (%sa) = map { $_ => 1; } (Cobalt::Vsite::vsite_admin_list("$mygr"));
	unless (exists($sa{"$uname"}) || ("$uname" eq "$Adminuser") || ("$uname" eq "root")) {
	    print "location: /.cobalt/error/forbidden.html\n\n";
	    exit 0;
	}

    } else {

	if (("$uname" ne "root") || ("$uname" ne "$Adminuser")) {
	    print "location: /.cobalt/error/forbidden.html\n\n";
	    exit 0;
	}

	$sch = "/scheduled-backup.admin";
    }

    # &build_backup();

    # Command line scheduled
    # s=100 new daily schedule
    # s=200 new weekly schedule
    # s=300 new monthly schedule
    # s=420 skip all this schedule crap

    # delete all
    @crons = ( $crondaily, $cronweekly, $cronmonthly );
    foreach $i ( @crons ) {
	my $sfile = "$i"."$sch";
	if (-e $sfile)
	{
		$existed = 1;
		unlink $sfile;
	}
    }

    if ($sched eq '1') {
	if ($existed) { &success(MSG_get_ui("3626")); }
	else { &error(MSG_get_ui("3631")); }
    }

    my ($inc,$pass,$reso);
    if ($args{'increment'}  =~ /^\d+$/) { $inc = '-i '.$args{'increment'}; } else { $inc = ''; }
    if ($args{'password'}) { $pass = '--password \''.$args{'password'}.'\''; } else { $pass = ''; }
    if ($args{'protocol'} eq 'smb') {
	$args{'resource'} = $smbinfo{'username'}.'@'.'\\\\\\\\'.$smbinfo{'host'}.'\\\\'.$smbinfo{'share'};
	$reso = $smbinfo{'username'}.'@'.'\\\\'.$smbinfo{'host'}.'\\'.$smbinfo{'share'};
    }

    @cmd = ($me, '-b', $args{'backupset'}, '-t', $args{'target'}, '-p', $args{'protocol'},
	       '-r', '"'.$args{'resource'}.'"', '-s 420', $inc, $pass );
    if ($args{'protocol'} eq 'smb') {
	$args{'resource'} = $reso;
    }

    $scheduleheader = <<EOF;
# Cobalt Networks
# Scheduled Backup
#
# backupset=$args{'backupset'}
# target=$args{'target'}
# protocol=$args{'protocol'}
# resource=$args{'resource'}
# increment=$args{'increment'}
# scheduled=$sched

EOF

    # put the new one in place
    if ($sched eq '100') {
	$schedule = "$crondaily"."$sch";
	open( SB, "> $schedule" );
	print SB $scheduleheader;
	print SB "@cmd\n";
	close SB;
	chown 0, 0, $schedule;
	chmod 0700, $schedule;
    }
    elsif ($sched eq '200') {
	$schedule = "$cronweekly"."$sch";
	open( SB, "> $schedule" );
	print SB $scheduleheader;
	print SB "@cmd\n";
	close SB;
	chown 0, 0, $schedule;
	chmod 0700, $schedule;
    }
    elsif ($sched eq '300') {
	$schedule = "$cronmonthly"."$sch";
	open( SB, "> $schedule" );
	print SB $scheduleheader;
	print SB "@cmd\n";
	close SB;
	chown 0, 0, $schedule;
	chmod 0700, $schedule;
    }

    &success(MSG_get_ui("3625"));
}

# # # # # # # # # # #
#                   #
#      H T T P      #
#                   #
# # # # # # # # # # #
if ( $args{'protocol'} eq 'http' )
{
    &build_backup();
    
    # Web dump
    print "Content-type: application/CobaltBackupArchive\n";
    print "Content-disposition: filename=$filename\n\n";

    print $header;
    print @index;

    open(TARR,"$tarcmd $filelist 2> /dev/null |");
    while(<TARR>) { print; }
    print TARR "\n";
    close(TARR);

    &cleanup;
    exit 0;
    &success(MSG_get_ui("3630"));
}

# # # # # # # # # # #
#                   #
#       N F S       #
#                   #
# # # # # # # # # # #
if ( $args{'protocol'} eq 'nfs' )
{
    &nfs_mount;
    &build_backup;

    my $backupfile = $mountpoint.'/'.$filename;

    open(OUTFILE,"> $backupfile") || &error(MSG_get_ui("3601"));
    print OUTFILE $header;
    print OUTFILE @index;
    open(TARR,"$tarcmd $filelist 2> /dev/null |");
    while(<TARR>) { print OUTFILE; }
    close TARR;
    close OUTFILE;
  
    &nfs_umount;

    &cleanup;

    &success(MSG_get_ui("3627"));
}

# # # # # # # # # # #
#                   #
#       S M B       #
#                   #
# # # # # # # # # # #
if ( $args{'protocol'} eq 'smb' )
{
    &build_backup;

    my $backupfile = $temp.'/'.$filename;

    open(OUTFILE,"> $backupfile") || &error(MSG_get_ui("3601"));
    print OUTFILE $header;
    print OUTFILE @index;
    open(TARR,"$tarcmd $filelist 2> /dev/null |");
    while(<TARR>) { print OUTFILE; }
    close TARR;
    close OUTFILE;

    chdir $temp;
    my @smbcmd = ( $Smbclient, "\\\\\\\\$smbinfo{'host'}\\\\$smbinfo{'share'}",
		   $smbinfo{'password'}, "-U $smbinfo{'username'}", "\"-c put $filename\"" );
    my $ret = `@smbcmd`;

    &error(MSG_get_ui("3602")) if ($ret);

    &cleanup;
    &success(MSG_get_ui("3628"));
}

# # # # # # # # # # #
#                   #
#       F T P       #
#                   #
# # # # # # # # # # #
if ( $args{'protocol'} eq 'ftp' ) {
    &build_backup;

    my $backupfile = $temp.'/'.$filename;

    open(OUTFILE,"> $backupfile") || &error(MSG_get_ui("3601"));
    print OUTFILE $header;
    print OUTFILE @index;
    open(TARR,"$tarcmd $filelist 2> /dev/null |");
    while(<TARR>) { print OUTFILE; }
    close TARR;
    close OUTFILE;

    my $ftp = Net::FTP->new($ftpinfo{'host'}) || &error(MSG_get_ui("3603", $ftpinfo{'host'}));
    $ftp->login($ftpinfo{'username'},$ftpinfo{'password'}) || &error(MSG_get_ui("3604"));
    $ftp->binary;
    if ($ftpinfo{'path'}) { $ftp->cwd($ftpinfo{'path'}) || &error(MSG_get_ui("3605")); }
    my $ret = $ftp->put($backupfile, $filename);
    &error(MSG_get_ui("3606")) unless ($ret eq $filename);
    $ftp->quit;

    &cleanup;
    &success(MSG_get_ui("3629"));
}

# everything else failed so show the command line help
&usage;

sub build_backup
{
    my ($file,$files,$dir);

    my @time = localtime;
    @time[4]++;
    $time[5] += 1900;
    $time[1] = '0'.$time[1] if ($time[1] !~ /\d\d/);
    $time[2] = '0'.$time[2] if ($time[2] !~ /\d\d/);
    $time[4] = '0'.$time[4] if ($time[4] !~ /\d\d/);
    $time[3] = '0'.$time[3] if ($time[3] !~ /\d\d/);

    my($d)=@time[5].@time[4].@time[3].'-'.@time[2].@time[1];
    my @xml;
    if ($args{'backupset'} eq 'site') {
	my(@vinfo)=Cobalt::Vsite::vsite_get_bygroup($args{'target'});
	$filename = "@vinfo[1]"."_site_$d$ext";
	@xml = &build_xml_info_vsite($args{'target'});
    } elsif ($args{'backupset'} eq 'user') {
	$filename = "$args{'target'}_$d$ext";
	@xml = &build_xml_info("users", $args{'target'});
    } else {
	my($h)=$args{'mygroup'};
	$h="$MainGroup" unless ($h);
	my(@vinfo)=Cobalt::Vsite::vsite_get_bygroup($h);
	$filename = "@vinfo[1]"."_$args{'target'}_$d$ext";
	my @sites = Cobalt::Meta::query("type" => "vsite", "keys" => ["name"]);
	@xml = &build_xml_info_vsite(@sites);
    }
    $header = &build_header;
    &build_file_table;

    push @index, "%%BACKUP_XML\n";
    foreach my $x (@xml) {
	push @index, $x;
	foreach my $y (@{ $x }) {
	    push @index, $y;
	}
    }
    push @index, "%%END_XML\n";

    open( FILELIST, "> $filelist" );
    foreach $file ( keys %files ) {
	print FILELIST "$file\n";
    }
    close FILELIST;

}

sub usage
{
    printf( "\nCobalt Backup/Restore version $version\n" );
    printf( "(c) 1998 Cobalt Networks, Inc.\n" );
    printf( "Duncan Laurie [duncan\@cobaltnet.com]\n\n" );
    printf( "Usage:  backup -b <string> -t <string> -p <string> -r <string> -i NUM -s NUM\n\n" );
    printf( "  -b, --backupset user|group|complete|mail     the backup set to use\n" );
    printf( "  -t, --target <user>|<group>|all       target for backupset\n" );
    printf( "  -p, --protocol http|smb|nfs                  the protocol to use\n" );
    printf( "  -r, --resource <string>                       nfs: hostname:/nfsresource\n" );
    printf( "                                                smb: login\\\\windowspc\\share\n" );
    printf( "                                                ftp: user\@host.domain.com/path\n" );
    printf( "  --password <string>                          password for resource\n" );
    printf( "  -i, --incremental NUM                        incremental [0 = no, NUM in days]\n" );
    printf( "  -s, --scheduled NUM                          scheduled backups\n" );
    printf( "  -h, --help                                   print this help text\n\n" );
    exit 2;
}

# constructs an xml list
# returns array of strings containing xml data
sub build_xml_info
{
    my $type = shift;
    my (@args) = @_;
    my (@xml,$x,$info);
    my $obj = Cobalt::Meta->create("type" => "$type");

    foreach $x (@args) {
	# get XML block
	$obj->retrieve($x);
	$info = $obj->as_xml();

	# save in array
	push @xml, $info;
    }

    return @xml;
}

# constructs an xml list of the vsites and corresponding users
# returns array of strings containing xml data
sub build_xml_info_vsite
{
    my (@vsites) = @_;
    my (@xml,@userxml,@users,$info,$site);
    my $Vobj = Cobalt::Meta->create("type" => "vsite");

    foreach $site (@vsites) {
	# get xml for site
	$Vobj->retrieve($site);
	$info = $Vobj->as_xml();

	# get xml for users, store as array
	@users = Cobalt::Vsite::vsite_get_users($site);
	@userxml = &build_xml_info("users", @users);
	# add xml for site to front of array
	unshift @userxml, $info;

	# save into 2D array
	push @xml, @userxml;
    }

    return(@xml);
}

sub build_file_table
# Constructs double hash list
# into %files{ fname, name, dir, mode, user, group, mode, mtime, size, md5checksum }
{
    my ($root,$findargs,$path,$fullname,$md5,@users,$user);

    $root = $backup_sets{$args{'backupset'}};

    if (($args{'increment'}  =~ /^\d+$/) && ($args{'increment'} ne '0'))
    {
	$findargs .= " -mtime -$args{'increment'}";
    }

    $findargs .= " -printf \"\\\"%f\\\" \\\"%h\\\" %m %u %g %s %Tc\\n\"";

    foreach $path ( $root )
    {
	push @index, "%%BACKUP_INDEX\n";
	open FSTREAM, "find $path $findargs 2> /dev/null |";
	while (<FSTREAM>)
	{
	    next unless /^\"(.*?)\" \"(.*?)\" (\S+) (\S+) (\S+) (\S+) (.*)\n/;

	    $fullname = "$2/$1";
	    if ( -d $fullname ) 
	    {
		$md5 = "0";
	    } else {
		next if ($exemptFiles{$fullname});
		next if $fullname =~ /^$Vol_rootdir\/\S+\/(quota.(?:user|group)|\.cobaltsig)/o;
		$md5 = `md5sum \"$fullname\" 2> /dev/null`;
		$md5 =~ /^(\S+)  .*$/;
		$md5 = $1;
	    }

	    %{ $files{$fullname} } = ( 'fname'   => $fullname,
				       'name'    => $1,
				       'path'    => $2,
				       'mode'    => $3,
				       'user'    => $4,
				       'group'   => $5,
				       'size'    => $6,
				       'mtime'   => $7,
				       'md5'     => $md5 );
	    push @index, "\'$2\' \'$1\' $3 $4 $5 $6 $md5 $7\n";
	}
	close FSTREAM;
	push @index, "%%END_INDEX\n";
    }

    &error(MSG_get_ui("3607")) unless (%files);

    return 0;
}

sub build_header
# Constructs archive description
# Arguments: None, uses global %args
# Return Value: header string
{
    my($user,$build,$hostname,$date,$datef);
    my($targetip,$targethost,$script,$browser,$serveradmin);
    
    $hostname=`hostname`;
    chomp( $hostname );
    $hostname ||= $ENV{'SERVER_NAME'};
    
    $user = $remote_user;
    $user ||= $args{'target'};
    &error(MSG_get_ui("3608")) unless ($user);
    
    if (-e "/etc/build")
    {
	open(BLD,"/etc/build");
	chomp($build = <BLD>);
	close(BLD);
    }
    
    $date = join(' ',localtime);
    $datef = scalar localtime;
    
    $targetip = $ENV{'REMOTE_ADDR'} || "commandline";
    $targethost = $ENV{'REMOTE_HOST'} || "commandline";
    $script = $ENV{'SCRIPT_NAME'} || "commandline";
    $browser = $ENV{'SCRIPT_NAME'} || "commandline";
    $serveradmin = $ENV{'SERVER_ADMIN'} || "commandline";

    my($header) = <<EOF;
%%BACKUP_HEADER
datef=$datef
build=$build
user=$user
backupset=$args{'backupset'}
target=$args{'target'}
increment=$args{'increment'}
scheduled=$args{'scheduled'}
protocol=$args{'protocol'}
resource=$args{'resource'}
hostname=$hostname
%%END_HEADER
EOF
    return($header);
}


sub parseinfo_cmdline
{
    my @ARGV = @_;
    my $ret;
    my %opt;

    $ret = GetOptions( \%opt, "backupset|b=s", "target|t=s", "protocol|p=s",
                       "resource|r=s", "increment|i=s", "help|h",
		       "scheduled|s=s", "password=s" );

    &error(MSG_get_ui("3609")) if ($ret == 0);

    return %opt;
}

sub parseinfo_web
{
    my %opt;
    my $q = new CGI( @_ );

    $opt{'backupset'} = $q->param('backupset');

    for( $opt{'backupset'} ) {
	/user/
	    and do {
		$opt{'target'} = $q->param('targetu');
		last;
	    };
	/group/
	    and do {
		$opt{'target'} = $q->param('targetg');
		last;
	    };
	/site/
	    and do {
		$opt{'target'} = $q->param('targets');
		last;
	    };
	$opt{'target'} = ''; # else
    }

    $opt{'protocol'} = $q->param('protocol');
    $opt{'resource'} = $q->param('resource');
    $opt{'increment'} = $q->param('increment');
    $opt{'scheduled'} = $q->param('scheduled');
    $opt{'password'} = $q->param('password');
    $opt{'password2'} = $q->param('password2');
    $opt{'mygroup'} = $q->param( 'group' );
    
    return %opt;
}

sub check_perms
{
    my $set = $args{'backupset'};
    my $tgt = $args{'target'};
    my $url = "/.cobalt/error/forbidden.html";

    my ($uid_r,$ru);
    if (defined($remote_user)) {
	$ru=$remote_user;
	$uid_r=(getpwnam($remote_user))[2];
    } else {
	$uid_r=$>;
	$ru=(getpwuid($uid_r))[0];
    }

    # admin gets away with everything
    return if(("$ru" eq "$Adminuser") || ("$ru" eq "root"));

    # site level ckecks
    if ($set eq "site") {

	# only let site admins backup up this site
	my (%site_admins) = map { $_ => 1; } (Cobalt::Vsite::vsite_admin_list("$tgt"));

	unless (exists($site_admins{"$ru"})) {
	    print "location: $url\n\n";
	    exit 0;
	}

    # user level checks
    } elsif ($set eq "user") {

	# if the user isn't backing themselves up...
	if ($ru ne $tgt) {

	    # dont let anyone but admin back up admin
	    if ($tgt eq $Adminuser) {
		print "location: $url\n\n";
		exit 0;
	    }

	    my $tgt_site = (Cobalt::User::user_list_site("$tgt"))[1];
	    my (%site_admins) = map { $_ => 1; } (Cobalt::Vsite::vsite_admin_list("$tgt_site"));

	    # error if not a site administrator for that site
	    unless (exists($site_admins{"$ru"})) {
		print "location: $url\n\n";
		exit 0;
	    }
	} else {
	    # if a user doing a (non scheduled) user backup, set effective UID/GID
	    unless (defined($args{'scheduled'})) {
		$< = $> = (getpwnam($ru))[2];
		$( = $) = (getpwnam($ru))[3];
	    }
	}

    # complete & config level checks
    } elsif (($set eq "config") || ($set eq "complete") || ($set eq "group")) {

	# only admin or root allowed to do these
	unless (($ru eq $Adminuser) || ($uid_r == 0)) {
	    print "location: $url\n\n";
	    exit 0;
	}
    }
}

sub check_options
{
    my ($myroot,@users,@user_stats,$u);
    my $mail;

    return if ($args{'scheduled'} eq '1');

    # passwords must match
    if ($args{'password'} ne $args{'password2'} && $args{'scheduled'} ne '420') { error(MSG_get_ui("passwordsNotMatch")); }

    # backupset must be specified
    $_ = $args{'backupset'};
  SWITCH: {
      if( /complete/ ) {
	  $args{'target'} = "complete";
	  last SWITCH;
      }
      if( /config/ ) {
	  $args{'target'} = "config";
	  last SWITCH;
      }
      if( /usergroup/ ) {
	  $args{'usergroup'} = "usergroup";
	  if (opendir(DIR, $Htgroup_dir)) {
	      while ($_ = readdir(DIR)) {
		  next unless /^site\d+/;
		  next unless -l "$Htgroup_dir/$_";
		  $backup_sets{'usergroup'} .= readlink("$Htgroup_dir/$_") . ' ';
	      }
	      closedir(DIR);
	  }
	  
	  if (opendir(DIR, $Mailspool)) {
	      while ($_ = readdir(DIR)) {
		  next if /^./;
		  next unless -l "$Mailspool/$_";
		  $backup_sets{'usergroup'} .= readlink("$Mailspool/$_") . ' ';
	      }
	      closedir(DIR);
	  }

	  last SWITCH;
      }

      if( /user/ ) {
	  @user_stats = getpwnam( $args{'target'} );
	  &error(MSG_get_ui("3612")) unless ( -d @user_stats[7] );
	  $backup_sets{'user'} .= @user_stats[7].' ';
	  $mail = $Mailspool . '/' . $args{'target'};
	  if (-e $mail) {
	      $backup_sets{'user'} .= "$mail ";
	      $backup_sets{'user'} .= readlink($mail) . ' ' if (-l $mail);
	  }
	  my($uservac)=$Vaca_dir.'/'.$args{'target'}.'.msg';
	  $backup_sets{'user'} .= "$uservac " if ( -e $uservac );
	  error(MSG_get_ui("3613")) unless ($backup_sets{'user'});
	  last SWITCH;
      }
      if( /group/ ) {
	  $myroot = $Htgroup_dir.'/'.$args{'target'};
	  if (-d $myroot) {
	      $backup_sets{'group'} .= "$myroot ";
	      $backup_sets{'group'} .= readlink($myroot)." " if (-l $myroot);
	  }
	  &error(MSG_get_ui("3613")) unless ($backup_sets{'group'});
	  last SWITCH;
      }
      if( /site/ ) {
	  $myroot = $Userdir.'/'.$args{'target'};
	  if (-d $myroot) {
	      $backup_sets{'site'} .= "$myroot ";
	      $backup_sets{'site'} .= readlink($myroot)." " if (-l $myroot);
	  }
	  @users = Cobalt::Vsite::vsite_get_users($args{'target'});
	  if ($users[0] =~ /^[134]\S*\s/) {
	      error(substr($users[0], 5));
	  } else {
	      foreach $u (@users) {
		  $mail =  "$Mailspool/$u";
		  if (-e $mail) {
		      $backup_sets{'site'} .= "$mail ";
		      $backup_sets{'site'} .= readlink($mail) . ' ' 
			  if (-l $mail);
		  }
		  $backup_sets{'site'} .= $Vaca_dir.'/'.$u.".msg "
		      if (-e "$Vaca_dir/$u.msg");
	      }
	  }
	  &error(MSG_get_ui("3613")) unless ($backup_sets{'site'});
	  last SWITCH;
      }
      if( // ) {
	  &error(MSG_get_ui("3610"));
	  last SWITCH;
      }
  }

    # target must be specified
    &error(MSG_get_ui("3611")) if ( $args{'target'} eq '' );

    # check to see if we were passed a valid ftp resource
    # user:pass@host.domain.com/path
    if ($args{'protocol'} eq 'ftp')
    {
	my ($username,$host,$path,$password);
	my $hostname = `hostname`;
	chomp($hostname);
	&error(MSG_get_ui("3614")) unless ($args{'resource'});
	if ( $args{'resource'} =~ /(\w+)@([\w|-|.]+)\/([\S]*)/ ) {
	    $username = $1;
	    $password = $args{'password'};
	    $host = $2;
	    $path = "/$3";
	} elsif ( $args{'resource'} =~ /(\w+)@([\w|-|.]+)/ ) {
	    $username = $1;
	    $host = $2;
	    $path = '';
	    $password = $args{'password'};
	} elsif ( $args{'resource'} =~ /([\w|.|-]+)([\w|\/|.|-]*)/ ) {
	    $username = 'anonymous';
	    $password = "user\@$hostname";
	    $host = $1;
	    $path = $2;
	} else { &error(MSG_get_ui("3615")); }
	%ftpinfo = ( 'username'  => $username,
		     'password'  => $password,
		     'host'      => $host,
		     'path'      => $path );
	&ftp_test();
    } elsif ($args{'protocol'} eq 'nfs') {
	&error(MSG_get_ui("3616")) unless ($args{'resource'});
	&error(MSG_get_ui("3617")) unless ( $args{'resource'} =~ /(\w+):\/(\w+)/ );
	nfs_test();
    } elsif ($args{'protocol'} eq 'smb') {
	&error(MSG_get_ui("3618")) unless ($args{'resource'});
	if ($args{'resource'} =~ /(\w+)@\\\\(\w+)\\(\w+)/ ) {
	    %smbinfo = ( 'username'   => $1,
			 'password'   => $args{'password'},
			 'host'       => $2,
			 'share'      => $3 );
	} elsif ($args{'resource'} =~ /\\\\(\w+)\\(\w+)/ ) {
	    %smbinfo = ( 'username'   => 'guest',
			 'password'   => $args{'password'},
			 'host'       => $1,
			 'share'      => $2 );
	} else { &error(MSG_get_ui("3619")); }
	smb_test();
    }
}

sub nfs_test
{
    nfs_mount();
    nfs_umount();
}

sub nfs_mount
{
    # first check and see if something is mounted on the temp mount location
    while ( -d $mountpoint ) {
	# mountpoint already exists, pick another temp
	$mountpoint .= "x";
    }

    mkdir $mountpoint, oct(0700) || &error(MSG_get_ui("3620"));

    my $mountcmd = $Mount_cmd.' '.$args{'resource'}.' '.$mountpoint.' 2> /dev/null';
    system( $mountcmd );
    unless ( ( -d $mountpoint ) && ( `$Mount_cmd | grep -c $mountpoint` == 1 ) ) {
	rmdir $mountpoint;
        &error(MSG_get_ui("3621"));
    }
}

sub nfs_umount
{
    &error(MSG_get_ui("3622")) unless ( -d $mountpoint );

    my $umountcmd = $Umount_cmd.' '.$mountpoint.' 2> /dev/null';
    system( $umountcmd );

    &error(MSG_get_ui("3623")) unless ( `$Mount_cmd | grep -c $mountpoint` );

    # umount successful, now remove the EMPTY directory
    rmdir $mountpoint || &error(MSG_get_ui("3623"));
}

sub smb_test
{
    my $smbcmd = "$Smbclient \\\\\\\\$smbinfo{'host'}\\\\$smbinfo{'share'} $smbinfo{'password'} -U $smbinfo{'username'} -c exit 2> /dev/null";
    my $ret = `$smbcmd`;
    error(MSG_get_ui("3624")) if ($ret);
}

sub ftp_test
{
    my $ftp = Net::FTP->new($ftpinfo{'host'}) || &error(MSG_get_ui("3603"), $ftpinfo{'host'});
    $ftp->login($ftpinfo{'username'},$ftpinfo{'password'}) || &error(MSG_get_ui("3604"));
    $ftp->cwd($ftpinfo{'path'}) || &error(MSG_get_ui("3605"));
    $ftp->quit;
}

sub archive_too_big
{
	my($comments) = @_;

        $ENV{'QUERY_STRING'} = 'USERERROR=The selected archive set is too large to be reliably backed up over the web. <BR><CENTER>Please select a smaller backup set, or use an advanced backup tool such as Legato NetWorker.  <BR>You may backup files individually using ftp.</CENTER>'.$comments.'#';
        open(MNU,"/usr/admserv/cgi-bin/.cobalt/backup/backupmenu.cgi |");
        while(<MNU>) { print; }
        close(MNU);

	exit 0; # Ugly exit, not proud of this
}

sub get_home_size
{
	my($homesize) = 0;

	open(SDF,"/bin/df |") || return 0;
	while(<SDF>)
	{
		my(@partinfo) = split(/\s+/);
		$homesize = @partinfo[2] if (@partinfo[5] eq '/home');
	}

	return $homesize;
}                 

sub cleanup
{
    unlink $filelist if ( -f $filelist );
    my $backupfile = $temp.'/'.$filename;
    unlink $backupfile if ( -f $backupfile );
}

sub success
{
    my @status = @_;

    if ($args{'scheduled'} eq '420') {

	my $locale = Cobalt::Locale->new();
	my $emailProp = $locale->getProperty("toolsModule");
	if ($emailProp) {
	    require "$emailProp";
	}

	my $subject  = "From: $Adminuser\n";
	$subject .= "To: $AlertList\n";

	if ($emailProp = ($locale->getProperty("email_ContentType"))) {
	    $subject .= "Content-Type: $emailProp\n";
	}
	
	if ($emailProp = ($locale->getProperty("email_ContentEncoding"))) {
	    $subject .= "Content-Transfer-Encoding: $emailProp\n";
	}
	
	$subject .= "Subject: Scheduled backup completed\n\n";

	my $body;
	if ($emailProp = ($locale->getProperty("email_ConvertMessage"))) {
	    $body = &$emailProp("@status");
	}
	$body .= "\n\nFilename: $filename\n\n$header\n";

	if (open (MAIL, "|$Sendmail -oi -t"))
	{
	    print MAIL $subject;
	    print MAIL $body;
	    close MAIL;
	}

    } elsif ($web) {

	&cleanup();

	my $qstr = "GROUP=".$args{'mygroup'}.'&'."USERSUCCESS="."@status".'#';
	$ENV{"QUERY_STRING"} = $qstr;
	my $url = "/usr/admserv/cgi-bin/.cobalt/backup/backupmenu.cgi";

	require("$url");

    } else {

	print "@status\n";

    }

    &cleanup();
    exit 0;
}

sub error
{
    my @error = @_;
    my $s = $args{'scheduled'};

    if ($s eq '420') {

	my $locale = Cobalt::Locale->new();
	my $emailProp = $locale->getProperty("toolsModule");
	if ($emailProp) {
	    require "$emailProp";
	}

	my $subject  = "From: $Adminuser\n";
	$subject .= "To: $AlertList\n";

	if ($emailProp = ($locale->getProperty("email_ContentType"))) {
	    $subject .= "Content-Type: $emailProp\n";
	}
	
	if ($emailProp = ($locale->getProperty("email_ContentEncoding"))) {
	    $subject .= "Content-Transfer-Encoding: $emailProp\n";
	}
	
	$subject .= "Subject: Scheduled backup failed\n\n";

	my $body;
	if ($emailProp = ($locale->getProperty("email_ConvertMessage"))) {
	    $body = &$emailProp("@error");
	}
	$body .= "\n\n";
	$body .= "backupset: $args{'backupset'}\n" if ($args{'backupset'});
	$body .= "target: $args{'target'}\n" if ($args{'target'});
	$body .= "protocol: $args{'protocol'}\n" if ($args{'protocol'});
	$body .= "resource: $args{'resource'}\n" if ($args{'resource'});
	$body .= "increment: $args{'increment'}\n" if ($args{'increment'});

	if (open (MAIL, "|$Sendmail -oi -t"))
	{
	    print MAIL $subject;
	    print MAIL $body;
	    close MAIL;
	}

    } elsif ($web) {

	&cleanup();

	my $qstr = "GROUP=".$args{'mygroup'}.'&'."USERERROR="."@error".'#';
	$ENV{"QUERY_STRING"} = $qstr;

	my $url = "/usr/admserv/cgi-bin/.cobalt/backup/backupmenu";
	$url .= 'S' if (($s eq '100') || ($s eq '200') || ($s eq '300'));
	$url .= ".cgi";

	require("$url");

    } else {

	print "@error\n";

    }

    &cleanup();
    exit 0;
}
