#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 3.0
# date: 6-28-99
#
# Version 2.0 uses Monitor.pm instead of AlertDB.pm
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::Meta;
use Cobalt::Util;
use Cobalt::swatch;

use strict;
use CGI;
use Locale::gettext;

#
# global variables
#
my $q=new CGI;

my $alertScriptURL='/cgi-bin/.cobalt/alert/alert.cgi';

my $cpuUsageTemplatePath="$Template_dir/cpuUsage.tmpl";

my $page=$q->param( 'page' );

if( $page ne 'cpuusagemain' ) { genPage(); }
else {
    print "content-type: text/plain\nCache-Control: no-cache\n\n";

    # read load average
    open( LOADAVG, "/proc/loadavg" ) || die "Can't read loadavg: $!\n";
    my $loadAvg1Min=( split( /\s+/, <LOADAVG> ) )[ 0 ];
    close LOADAVG;

    print "loadavg1: $loadAvg1Min\n";
}


sub genPage {
    my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );

    my ($cpu,$defcon,$ball,$message,$load15,$info,$back,$procs);
    my ($uptime,$up_hrs,$up_min,$up_day);
    my ($msg_d,$msg_h,$msg_m);
    my $temp;

    $cpu = Cobalt::Meta->create("type" => "cpu");
    $cpu->retrieve("cpu");
    $defcon = swatch_get_defcon("cpu");
    if ($defcon) {
	$ball = $defcon_images{ $defcon };
	$message = MSG_get_ui("swatch_cpu_body_defcon_$defcon");
	
	$temp = $cpu->get("temperature");
	$temp = ($temp) ? $temp : 0;

	$load15 = $cpu->get("load15");
	$load15 = ($load15) ? $load15 : "0.00";
	
	$uptime = $cpu->get("uptime");
	$up_min = int($uptime / 60);
	$up_hrs = int($up_min / 60);
	$up_min = int($up_min % 60);
	$up_day = int($up_hrs / 24);
	$up_hrs = int($up_hrs % 24);

	$msg_d = ($up_day == 1) ? MSG_get_ui("swatch_cpu_uptime_day") : 
	    MSG_get_ui("swatch_cpu_uptime_days");	
	$msg_h = ($up_hrs == 1) ? MSG_get_ui("swatch_cpu_uptime_hour") : 
	    MSG_get_ui("swatch_cpu_uptime_hours");	
	$msg_m = ($up_min == 1) ? MSG_get_ui("swatch_cpu_uptime_minute") : 
	    MSG_get_ui("swatch_cpu_uptime_minutes");	

	$info = "<BR>" . MSG_get_ui("swatch_cpu_load15", $load15) . "<BR>";
	$info .= MSG_get_ui("swatch_cpu_info", $temp) . "<BR>\n";
	$info .= MSG_get_ui("swatch_cpu_uptime") . "  " .
	        $up_day . $msg_d . $up_hrs . $msg_h . 
		MSG_get_ui("swatch_cpu_uptime_and") . $up_min . $msg_m;
	
    } else {
	$ball = $defcon_images{ 0 };
	$message = MSG_get_ui("swatch_body_defcon_0");
    }

    $back='';
    if ($q->param('alert')) {
	$back="<A HREF=\"$alertScriptURL\"><IMG SRC=\"/.cobalt/images/back_but.gif\" BORDER=0></A>";
    }

    open( TEMPLATE, "<$cpuUsageTemplatePath" )
	|| die "Cannot read $cpuUsageTemplatePath: $!\n";

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    while( <TEMPLATE> ) {
	s/\[DEFCON\]/$ball/;
	s/\[MESSAGE\]/$message/;
	s/\[INFO\]/$info/;
	s/\[BACK\]/$back/;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print;
    }

    close( TEMPLATE );
}


1;
