#! /usr/bin/perl

#
# Cobalt Diagnostic Dump
#
# Written by Michael Stone (mstone@cobaltnet.com)
# Adapted by Timothy Stonis (tim@cobaltnet.com)
#
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com
#

use Cobalt::Product;
use Cobalt::Locale;

use Locale::gettext;


use strict;
use CGI;


# global variable
my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

# Array consists of object and function pairs.
#
# The first element of a pair is the target object which can be a file,
# a file spec (including a valid shell regex), or a shell command.
#
# The second element of a pair is the function to run on the target.
# At this time the valid functions are as follows:
#
#	1 = cat the file to the dump
#	2 = diff the file to it's .master counterpart
#	3 = tail -1 the file (last line)
#	4 = tail -30 the file (a bit more)
#	5 = tail -50 the file (even more)
#	6 = Execute the command via a shell
#
# Why didn't I use a fancy hash instead of this array hack? Because I wanted
# to preserve the order of the elements.
#


my @ToDump = (  "/etc/build",1,
		"/etc/passwd",1,
		"/etc/group",1,
		"/etc/shadow",1,
		"/etc/gshadow",1,
		"/etc/aliases",1,
		"/etc/virtusertable",1,
		"/usr/sbin/repquota -a",6,
		"/etc/sendmail.cf",2,
		"/etc/sendmail.cw",1,
		"/etc/mail/access",1,
		"/etc/proftpd.conf",1,
		"/etc/ftpusers",1,
		"/etc/sysconfig/network-scripts/ifcfg-eth*",1,
		"/bin/ls -al /home/sites/",6,
		"/etc/httpd/conf/access.conf",1,
		"/etc/httpd/conf/httpd.conf",1,
		"/etc/httpd/conf/srm.conf",1,
		"/etc/named.conf",1,
		"/etc/named/records",1,
		"/bin/ls -lR /etc/named/*",6,
		"/var/cobalt/*.log",3,
		"/bin/ls -lR /var/log/*",6,
		"/var/log/messages",5,
		"/var/log/maillog",4,
		"/var/log/httpd/access",5,
		"/var/log/httpd/error",5,
		"/bin/uname -a",6,
		"/bin/ps aux",6,
		"/var/log/dmesg",1,
		"/usr/bin/w",6,
		"/sbin/ifconfig",6,
		"/sbin/route -n",6,
		"/bin/rpm -qa",6,
		"/etc/hosts.allow",1,
		"/etc/hosts.deny",1

	);



my $q=new CGI;
my $DumpFlag = $q->param( 'dump' );

# Location of the dump file before compression.
my $DiagDumpPath = "/home/tmp";
my $DiagDumpFile = "diagnostic.dump";
my $mode = 0600;
my $DiagMacAddr = &diag_get_mac;


# We do the dump if our $DumpFlag is greater than 1
# This is here so we can call the script directly and give it dump=1 to just
# dump the file without the http header and gzip.

if ($DumpFlag >= 1)
	{
		&diag_handle_form;

	#
	# if dump is not set (<1) then we output the form
	#

	} else {

		&diag_gen_form;

	}
1;



sub diag_handle_form
# diag_handle_form: Take form contents and create the diagnostic dump file
# for web download.
# Arguments: none
# Return value: none
# Side effects: none
# Example: &diag_handle_form;

{


	# If the file exists we delete it first.
	if (-e "$DiagDumpPath/$DiagDumpFile")
	{
		unlink "$DiagDumpPath/$DiagDumpFile" || die "Can't delete $DiagDumpFile";
	}
	system ("/bin/touch $DiagDumpPath/$DiagDumpFile");
	system ("/bin/chmod 0600 $DiagDumpPath/$DiagDumpFile");

	# First we write a header to our diag file from the form fields.
	&diag_put_header("Customer_Info_Start");

	open (DIAG, ">>$DiagDumpPath/$DiagDumpFile");

	print DIAG "Name: ".$q->param('ddname')."\n";
	print DIAG "Company: ".$q->param('ddcompany')."\n";
	print DIAG "Reply: ".$q->param('ddreply')."\n";
	print DIAG "Phone: ".$q->param('ddphone')."\n";
	print DIAG "Email: ".$q->param('ddemail')."\n";
	print DIAG "MacAddr: ".$DiagMacAddr."\n";
	print DIAG "Problem:\n".$q->param('ddproblem')."\n";

	close DIAG;

	&diag_put_header("Customer_Info_End");


	# Now we look at our list of things to do and do them
	while (@ToDump) {

		# The $target is either a filespec or a command.
		# The next element of the @ToDump array determines what we do 
		# with the previous element
	
		my $target = shift @ToDump;
		my $diagfunc = shift @ToDump;

		if ($diagfunc == 1) {
			&diag_cat_target($target);
		}
		elsif ($diagfunc == 2) {
			&diag_diff_target($target);
		}
		elsif ($diagfunc == 3) {
			&diag_tail_short($target);
		}
		elsif ($diagfunc == 4) {
			&diag_tail_long($target);
		}
		elsif ($diagfunc == 5) {
			&diag_tail_longest($target);
		}
		elsif ($diagfunc == 6) {
			&diag_exec_target($target);
		}
	
	} #End of While


	# If we are called by the UI form then $DumpFlag will = 2
	# and we should gzip the dump file and send it to the browser,
	# otherwise we're done.

	if ($DumpFlag == 2) {

		print "Content-type: application/CobaltDiagnosticDump\n"; # For ie4
		print "Content-disposition: filename=Diag\_RaQ\_$DiagMacAddr.gz\n\n";

		open(TR,"/bin/gzip -c $DiagDumpPath/$DiagDumpFile |");

		while(<TR>) { print; }
		close(TR);

		}
}


sub diag_gen_form
# diag_gen_form: Generate the UI form for the diagnostic dump user data.
# Arguments: none
# Return value: none
# Side effects: Sends HTML for the form.
#
# Example: $return = diag_diff_target($target);

{

open( TEMPLATE, "$Template_dir/diagDump.tmpl" )
    || die "Cannot read $Template_dir/diagDump.tmpl: $!\n";

while ( <TEMPLATE> ) {

    s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

    print;
}

close TEMPLATE;

}



sub diag_put_header
# diag_put_header: Writes a header for each element that is included in the dump
# file.
# Arguments: scalar value to write on header line for section
# Return value: none
# Side effects: none
# Example: $return = diag_put_header($some_scalar);
{

	open (DIAG, ">>$DiagDumpPath/$DiagDumpFile");

	my $headertext = shift(@_);
	print DIAG "CobaltDiag::$headertext\n";
	close DIAG;
}

sub diag_cat_target
# diag_cat_target: writes a files contents to diag dump
# Arguments: target file name
# Return value: none
# Side effects: none
# Example: $return = diag_cat_target($some_scalar);

{

	my ($target,$cmd) = @_;
	my @Diag_FileList = glob( $target );

	while (@Diag_FileList) {
		my $catfile = shift(@Diag_FileList);
		&diag_put_header("cat $catfile");
		system ("/bin/cat $catfile >> $DiagDumpPath/$DiagDumpFile");
	}
}


# Perform a diff on a file and it's master
sub diag_diff_target
# diag_diff_target: performs a diff of target to it's .master file and writes to diag dump.
# Arguments: target file name
# Return value: none
# Side effects: If the specified file name does not have a corresponding
# master file then the routine will produce an error but will not crash.
#
# Example: $return = diag_diff_target($target);
{

	my ($target,$cmd) = @_;
	my @Diag_FileList = glob( $target );


	while (@Diag_FileList) {
		my $catfile = shift(@Diag_FileList);
		&diag_put_header("diff $catfile $catfile.master");
		system ("/usr/bin/diff $catfile $catfile.master >> $DiagDumpPath/$DiagDumpFile");
	}

}

sub diag_tail_short
# diag_tail_short: Writes the last line of a file to diag dump.
# Arguments: target file name 
# Return value: none
# Side effects: none
# Example: $return = diag_tail_short($some_scalar);

{

	my ($target,$cmd) = @_;
	my @Diag_FileList = glob( $target );

	while (@Diag_FileList) {
		my $catfile = shift(@Diag_FileList);
		&diag_put_header("tail -1 $catfile");
		system ("/usr/bin/tail -1 $catfile >> $DiagDumpPath/$DiagDumpFile");
	}

}

sub diag_tail_long
# diag_tail_long: Writes last 30 lines of a file to diag dump.
# Arguments: target file name 
# Return value: none
# Side effects: none
# Example: $return = diag_tail_long($some_scalar);

{

	my ($target,$cmd) = @_;
	my @Diag_FileList = glob( $target );

	while (@Diag_FileList) {
		my $catfile = shift(@Diag_FileList);
		&diag_put_header("tail -30 $catfile");
		system ("/usr/bin/tail -30 $catfile >> $DiagDumpPath/$DiagDumpFile");
	}

}

sub diag_tail_longest
# diag_tail_longest: Writes last 50 lines of a file to diag dump.
# Arguments: target file name
# Return value: none
# Side effects: none
# Example: $return = diag_tail_long($some_scalar);

 {

	my ($target,$cmd) = @_;
	my @Diag_FileList = glob( $target );

	while (@Diag_FileList) {
		my $catfile = shift(@Diag_FileList);
		&diag_put_header ("tail -50 $catfile");
		system ("/usr/bin/tail -50 $catfile >> $DiagDumpPath/$DiagDumpFile");
	}

}

sub diag_exec_target
# diag_exec_target: Executes a command with output to diag dump
# Show me the output of a command
#
# Arguments: command to execute in a scalar
# Return value: none
# Side effects: none
# Example: $return = diag_exec_target($some_scalar);

 {

	my ($target,$cmd) = @_;
	&diag_put_header ("$target");
	system ("$target >> $DiagDumpPath/$DiagDumpFile");

}


sub diag_get_mac
# diag_get_mac: Returns the MAC address of eth0 
# Arguments: none
# Return value: mac address of eth0 in standard format
# Side effects: none
# Example: $return = diag_get_mac;

{

	my $null;
	my $mac;

	open (IFCFG, "/sbin/ifconfig eth0 |");

	while (<IFCFG>) {
        	if (/HWaddr/) {
			s/\://g;
			chop;
			close IFCFG;
        		($null,$mac) = split(/HWaddr /);
			return $mac;
		}
	}

	close IFCFG;


} 

