#!/usr/bin/perl

# author: Kevin K.M. Chiu
#         Timothy Stonis
#       Cobalt Networks
# version: 2.0
# date: 7-2-99
#
# Copyright (C) 1999-2000, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::Admserv;
require Cobalt::Network;

use CGI;
use strict;
use Locale::gettext;

# global variables
my $scriptURL='/cgi-bin/.cobalt/register/register.cgi';

my $q=new CGI;

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $page=$q->param( 'page' );

if( $page ne 'registermain' ) {

   # First check to see if we registered already
   my $reg = (-e "/var/lib/cobalt/.registered") ? 1 : 0;
   if( $reg ) {
     genGeneralRegPage( "[str.alreadyRegistered]" ); 
     # print "location: /.cobalt/register/noRegister.html\n\n";
   } else {

      my( $dns1, $dns2 )=( Cobalt::Network::net_get_dnsclient() )[ 1, 2 ];
      my $gateway=( Cobalt::Network::net_get_gateway() )[ 0 ];

      if( ( !$dns1 && !$dns2 ) || !$gateway ) {
  	# cannot register without DNS or gateway to send email
	# print "location: /.cobalt/register/noRegister.html\n\n";
        genGeneralRegPage( "[str.noEmailRegister]" );	
      }
      else { genRegisterPage(); }
    }
}
else {
    # get user information

    #
    # some get used while some don't depending on locale
    #
    my $fullName=$q->param( 'fullname' );
    my $title=$q->param( 'title' );
    my $company=$q->param( 'company' );
    my $address=$q->param( 'address' );
    my $country=$q->param( 'country' );
    my $phone=$q->param( 'phone' );
    my $fax=$q->param( 'fax' );
    my $email=$q->param( 'email' );

    if( !$fullName ) {
	print "content-type: text/html\nCache-Control: no-cache\n\n";
	genRegisterPage( "<IMG SRC=\"/.cobalt/images/bang.gif\" ALIGN=CENTER> ".eval("\"".Locale::gettext::dgettext("ui", "noFullName")."\"")."" );
    }
    elsif( !$fax && !$email ) {
	print "content-type: text/html\nCache-Control: no-cache\n\n";
	genRegisterPage( "<IMG SRC=\"/.cobalt/images/bang.gif\" ALIGN=CENTER> ".eval("\"".Locale::gettext::dgettext("ui", "noFaxOrEmail")."\"")."" );
    }
    else {
	# get system information
	my $hostName=Cobalt::Network::net_get_hostname();
	my ( $domain, $dns1, $dns2 )=Cobalt::Network::net_get_dnsclient();
	my ( $ipAddr, $netMask, $mac )=Cobalt::Network::net_get_ipaddr();

	open( BUILD, '</etc/build' );
	my $build=join "\t", <BUILD>;
	close( BUILD );

	my $ProductName;
	if ($Features{'mgmt'}) {
		$ProductName = "ManageRaQ";
	} else  {
		$ProductName = "RaQ 3";
	}

	if( open( MAIL, "|$Sendmail -oi -t" ) ) {
	    print MAIL <<END;
From: $fullName <admin\@$hostName.$domain>, <$email>
To: register\@cobaltnet.com
Subject: Registration ($ProductName)

Full name: $fullName
Title: $title
Company: $company
Address: $address
Country: $country
Phone: $phone
Fax: $fax
Email: $email

IP address: $ipAddr
Netmask: $netMask
Host name: $hostName
Domain: $domain
DNS 1: $dns1
DNS 2: $dns2
MAC address: $mac

Build: $build
END
	}

        open( REGFILE, ">/var/lib/cobalt/.registered" );
        close ( REGFILE ); 
        genGeneralRegPage( "[str.successReg]" );	
    }
}

1;


#
# argument 0 is the infomation shown to user
#
sub genRegisterPage {
    # Read in the template
    open( TEMPLATE, "$Template_dir/register.tmpl" )
        || die "Cannot read $Template_dir/register.tmpl: $!\n";

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    while( <TEMPLATE> ) {
	s/\[SCRIPTURL\]/$scriptURL/o;
	s/\[USER_INFO\]/$_[ 0 ]/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print;
    }

    close TEMPLATE;

   # I'm going to be real nasty here... I don't want the user to have
   # to authenticate right after completing the first page of the setup
   # wizard, so I'll throw a .htaccess file in here after I present
   # the page.
   admserv_set_access( "$AdminPages_dir/register/", 'admin' );
   admserv_set_access( "$AdminCGIs_dir/register/", 'admin' );

}

#
# Argument 0 is the text you want to put in the dialog
#
sub genGeneralRegPage {
    # Read in the template
    open( TEMPLATE, "$Template_dir/genregister.tmpl" )
        || die "Cannot read $Template_dir/register.tmpl: $!\n";

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    while( <TEMPLATE> ) {
        s/\[REGMESSAGE\]/$_[ 0 ]/o;
        s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

        print;
    }

    close TEMPLATE;

   # I'm going to be real nasty here... I don't want the user to have
   # to authenticate right after completing the first page of the setup
   # wizard, so I'll throw a .htaccess file in here after I present
   # the page.
   admserv_set_access( "$AdminPages_dir/register/", 'admin' );
   admserv_set_access( "$AdminCGIs_dir/register/", 'admin' );

}

