#!/usr/bin/perl
# Copyright 1998-2000, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Admserv;
use Cobalt::Email;
use Cobalt::Ftp;
use Cobalt::Fpx;
use Cobalt::Group;
use Cobalt::List;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Network;
use Cobalt::Stats;
use Cobalt::Vsite;
use Cobalt::Http;
use Cobalt::Meta;
use Cobalt::Util;
use Cobalt::Product;
use Cobalt::DiskUtils;
use Cobalt::BandWidth;
use Cobalt::Layout;

use DB_File;
use Fcntl qw(O_RDONLY O_RDWR O_CREAT);
use strict;
use CGI;
use Locale::gettext;

# global variables
my $q=new CGI;

my $siteAddScriptURL="/cgi-bin/.cobalt/siteAdd/siteAdd.cgi";
my $siteAddScriptPath="$AdminCGIs_dir/siteAdd/siteAdd.cgi";
my $siteListScriptURL='/cgi-bin/.cobalt/siteList/siteList.cgi';
my $siteListScriptPath="$AdminCGIs_dir/siteList/siteList.cgi";

# the page that run this script
my $page = $q->param('page');
my $error = $q->param('error');

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

if ($error eq 'yes') {
    my $e_hostname = $q->param('hostname');
    my $e_domain = $q->param('domain');
    my $e_ipaddr = $q->param('ipaddr');
    my $e_emaildomain = $q->param('emaildomain');
    my $e_emailserveralias = $q->param('emailserveralias');
    my $e_webdomain = $q->param('webdomain');
    my $e_quota = $q->param('quota');
    my $e_maxusers = $q->param('maxusers');
    my $e_shell = $q->param('shell');
    my $e_cgi = $q->param('cgi');
    my $e_ssi = $q->param('ssi');
    my $e_ssl = $q->param('ssl');
    my $e_fpx = $q->param('fpx');
    my $e_ftp = $q->param('ftp');
    my $e_ftpusers = $q->param('ftpusers');
    my $e_ftpquota = $q->param('ftpquota');
    my $e_apop = $q->param('apop');
    my $e_bwlimit = $q->param('bwlimit');
    my $e_bw_setip = $q->param('bw_setip');
    my $e_info = $q->param('info');
    $e_info =~ s/[\n\r]/ /g; # returns break javascript

    my $errobj = Cobalt::Meta->new("type" => "vsite",
				   "name" => "error",
				   "hostname" => $e_hostname,
				   "domain" => $e_domain,
				   "ipaddr" => $e_ipaddr,
				   "emaildomain" => $e_emaildomain,
				   "emailserveralias" => $e_emailserveralias,
				   "webdomain" => $e_webdomain,
				   "quota" => $e_quota,
				   "maxusers" => $e_maxusers,
				   "shell" => $e_shell,
				   "cgi" => $e_cgi,
				   "ssi" => $e_ssi,
				   "ssl" => $e_ssl,
				   "fpx" => $e_fpx,
				   "ftp" => $e_ftp,
				   "ftpusers" => $e_ftpusers,
				   "ftpquota" => $e_ftpquota,
				   "apop" => $e_apop,
				   "bwlimit" => $e_bwlimit);
    
    my $errbwobj = Cobalt::Meta->new("type" => "bw",
				     "name" => "error",
				     "bwlimit" => $e_bw_setip);
    
    genSiteAddPage($errobj, $errbwobj, $e_info);
    
} elsif ($page eq 'siteadd') {

    handleSiteAddPage();

} else {

    my $defobj = Cobalt::Meta->create("type" => "vsite");
    $defobj->retrieve("default");
    my $defbwobj = Cobalt::Meta->create("type" => "bw");
    $defbwobj->retrieve("default");

    genSiteAddPage($defobj, $defbwobj);
}

1;

#
# event handler for add site page
#
sub handleSiteAddPage {
    my $herr=0;
    my $err=0;
    my $info='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';
    my $ret;

    # read form input
    my $hostName=lc($q->param('hostname'));
    my $domain=lc($q->param('domain'));
    my $ipAddr=$q->param('ipaddr');
    my $diskQuota=$q->param('diskquota');
    my $cgi=$q->param('cgi');
    my $shell=$q->param('shell');
    my $ssi=$q->param('ssi');
    my $ssl=$q->param('ssl');
    my $fpx=$q->param('fpx');
    my $bw_setip=$q->param('bw_setip');
    my $bwlimit=$q->param('bwlimit');
    my $ftp=$q->param('ftp');
    my $ftpQuota=$q->param('ftpquota');
    my $ftpUsers=$q->param('ftpusers');
    my $maxUsers=$q->param('maxusers');
    my $emailDomain=$q->param('emaildomain');
    my $emailsvrali=$q->param('emailserveralias');
    my $webDomain=$q->param('webdomain');
    my $apop=$q->param('apop');
    my $suspend=$q->param('suspend');

    $fpx ? ($fpx = 'on') : ($fpx = 'off');
    $ftp ? ($ftp = 'on') : ($ftp = 'off');
    $cgi ? ($cgi = 'on') : ($cgi = 'off');
    $ssi ? ($ssi = 'on') : ($ssi = 'off');
    $ssl ? ($ssl = 'on') : ($ssl = 'off');
    $apop ? ($apop = 'on') : ($apop = 'off');
    $suspend ? ($suspend = 'on') : ($suspend = 'off');
    $shell ? ($shell = 'on') : ($shell = 'off');
    $emailDomain ? ($emailDomain = 'on') : ($emailDomain = 'off');
    $webDomain ? ($webDomain = 'on') : ($webDomain = 'off');
    $bwlimit ? ($bwlimit = 'on') : ($bwlimit = 'off');
    $bw_setip = $bw_setip ? $bw_setip : 0;

    my $obj = Cobalt::Meta->new("type" => "vsite");

    my $volume = $q->param('volume');
    my $mountpoint = "$Vol_rootdir/$volume" unless ($volume eq $Vol_home);
    unless (defined($volume)) {
	$info = MSG_get_ui("volNameRequired");
	$herr = 1;
    }

    my $host = $hostName;
    if (netutil_test_domainname($domain) && 
        netutil_test_domainname($hostName)) {
	$domain =~ s/^\.//g; # strip leading .'s
	$hostName = "$hostName.$domain";
    } 
    else {
        $info.=eval("\"".Locale::gettext::dgettext("ui", "hostNameRequired")."\"");
	$herr=1;
    }

    # can't have vsite's fqdn as vsite's email server alias
    # because virtusertable will not rebuild.
    my @emailSvrAli = split(',', $emailsvrali);
    my ($alias);
    foreach $alias( @emailSvrAli )  {
        if($alias eq $hostName)  {
            $info.=eval("\"".Locale::gettext::dgettext("ui",
                "emailServerAliasInvalid")."\"") . "<BR>";
	    @emailSvrAli = ();
	    $emailsvrali = "";
            $herr=1;
        }
    }

    # if this is a name based site and they enabled bandwidth management or it is
    # already enabled for another virtual site, we need to warn them
    my @other_ipaddrs = Cobalt::Meta::query("type"  => "vsite",
					    "keys"  => ["name"],
					    "where" => ["name", "<>", "default",
							"and", "ipaddr", "=", "$ipAddr"]);

    # find out if the same fqdn is used
    my $other_fqdns = scalar(Cobalt::Meta::query("type"  => "vsite",
						 "keys"  => ["name"],
						 "where" => ["name", "<>", "default",
							     "AND", "hostname", "=", "$host",
							     "AND", "domain", "=", "$domain"]));

    if ($other_fqdns) {
	my($name)=$hostName;
	$info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_hostnameinuse")."\"") . "<BR>";
	$herr=1;
    }
    # find out if another site w/ same ip has ssl on
    if($ssl eq 'on') {
	if (scalar(Cobalt::Meta::query("type"  => "vsite",
				       "keys"  => ["name"],
				       "where" => ["name", "<>", "default",
						   "AND", "ipaddr", "=", "$ipAddr",
						   "AND", "ssl", "=", "on"]))) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_ipinuse_ssl")."\"") . "<BR>";
	    $herr=1;
	}
    }
    # find out if another site with the same ip has ftp on
    if($ftp eq 'on') {
	if (scalar(Cobalt::Meta::query("type"  => "vsite",
				       "keys"  => ["name"],
				       "where" => ["name", "<>", "default",
						   "AND", "ipaddr", "=", "$ipAddr",
						   "AND", "ftp", "=", "on"]))) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_ipinuse_ftp")."\"") . "<BR>";
	    $herr=1;
	}
    }
    # find out if another site with the same ip has emaildomain on
    if($emailDomain eq 'on') {
	if (scalar(Cobalt::Meta::query("type"  => "vsite",
				       "keys"  => ["name"],
				       "where" => ["name", "<>", "default",
						   "AND", "domain", "=", "$domain",
						   "AND", "emaildomain", "=", "on"]))) {
	    my $r = $domain;
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "email_cant_route_used")."\"") . "<BR>";
	    $herr=1;
	}
    }


    # find out if another site with the same ip has webdomain on
    if($webDomain eq 'on') {
	if (scalar(Cobalt::Meta::query("type"  => "vsite",
				       "keys"  => ["name"],
				       "where" => ["name", "<>", "default",
						   "AND", "domain", "=", "$domain",
						   "AND", "webdomain", "=", "on"]))) {
	    my $r = $domain;
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_webdomain")."\"") . "<BR>";
	    $herr=1;
	}
    }
    if( !$hostName || !$ipAddr || !$diskQuota ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_essential_data")."\"") . "<BR>";
	$herr=1;
    }
    if( $hostName=~/localhost/io ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "localhostNotAllowed")."\"") . "<BR>";
	$herr=1;
    }
    if( $hostName && !netutil_test_domainname( $hostName ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "hostNameNotValid")."\"") . "<BR>";
	$herr=1;
    }
    if( !netutil_test_ipaddr( $ipAddr ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "addrNotValid")."\"") . "<BR>";
	$herr=1;
    }
    if( $diskQuota!~/^\d+$/o || $diskQuota==0 ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotPositive")."\"") . "<BR>";
	$herr=1;
    }
    if(($ftp eq 'on') && $ftpQuota!~/^\d+$/o ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "ftpQuotaNotInteger")."\"") . "<BR>";
	$herr=1;
    }
    if( ( $ftp eq 'on' ) && $ftpQuota>$diskQuota ) {
	# for the message
	my $siteDiskQuota=$diskQuota;
	$info.=eval("\"".Locale::gettext::dgettext("ui", "ftpQuotaTooBig")."\"") . "<BR>";
	$herr=1;
    }
    if( ( $ftp eq 'on' ) && ( $ftpUsers!~/^\d+$/o || $ftpUsers==0 ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "ftpUsersNotPositive")."\"") . "<BR>";
	$herr=1;
    }
    if ($bw_setip < 10 && $bwlimit eq 'on') {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "bw_setip_invalid")."\"") . "<BR>";
	$herr=1;
    }
    if (!$herr) {

	# everything should be fine
	# if no bandwidth entered assume 0 
	defined($bw_setip) ? $bw_setip : 0;

	# name-based generation for filesystem readability
	my $groupName=vsite_gen_name( $hostName );

	# logs rotate in MB not blocks
	my $logRotateLimit=int($diskQuota/10);
	$logRotateLimit=1 unless ($logRotateLimit);

	# convert from megabyte to block
	my $quota = $diskQuota;
	$diskQuota*=1048576/$BytesPerBlock;
	my $fquota = $ftpQuota;
	$ftpQuota*=1048576/$BytesPerBlock;

	# make the group for the site
	$ret=group_add( $groupName, $diskQuota, $mountpoint );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

        Cobalt::Layout::layout_siteManage($groupName,$hostName);
	$ret=admserv_set_access( "$AdminPages_dir/siteManage/$groupName",
				 'require', 'group', $groupName );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
	
	# substitute variable names
	my $idxin = "$Htgroup_dir/$groupName/$Webdir/index.html";
	my $idxout = "/etc/locks/index.html.$groupName";
	if (-f $idxin) {
	    open(IN,"$idxin");
	    open(OUT,">$idxout");
	    while( <IN> ) {
		s/\[DOMAIN\]/$hostName/go;
		print OUT;
	    }
	    close( IN );
	    close( OUT );
	    system( "mv -f $idxout $idxin" );
	    chown($WWWuid,(getgrnam($groupName))[2],"$Htgroup_dir/$groupName/$Webdir/index.html");
	    chmod(0664,"$Htgroup_dir/$groupName/$Webdir/index.html"); 
	}

	# setup a virtual list server
	$ret=list_create_virtual ($groupName, $hostName);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# Missing optional webserver admin and subnet mask
	$ret=vsite_add( $ipAddr, $hostName, $groupName );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# Rotate split log fragments for the site
	$ret=vsite_set_logrotate($groupName,$logRotateLimit);
	
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	if ($webDomain eq 'on') {
	    # add a ServerAlias directive to the vhost section
	    $ret = vsite_add_alias($groupName, $domain);
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	if ($emailDomain eq 'on') {
	    $ret=Cobalt::Email::mail_virtuser_add_route($domain, $hostName);
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	    $ret=Cobalt::Email::mail_add_relay( $domain );
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4).'<BR>';
		$err=1;
	    }
	    $ret=Cobalt::Email::mail_add_cw( $hostName, $domain );
	} else {
	    $ret=Cobalt::Email::mail_add_cw($hostName);
	    $emailDomain='off';
	}
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# add IP address to relay list
	$ret=Cobalt::Email::mail_add_relay( $ipAddr );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# add default aliases
	$ret=Cobalt::Email::mail_virtuser_dfl_aliases( $hostName, 1);
        if ( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4).'<BR>';
	    $ret = 1;
        }

        Cobalt::Email::mail_sendmail_hup();
	
	# config ftp
	if ($ftp eq 'on') {
	    $ret=ftp_config($ipAddr,1,1,$groupName,$ftpUsers,$ftpQuota);
	} else {
	    $ret=ftp_config($ipAddr,1,0,$groupName);
	}
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

        # handle valid email server aliases
        my @goodal = ();
        my %emailal = Cobalt::Email::mail_virtuser_list_route();
        if (scalar(@emailSvrAli)) {
            foreach my $r (@emailSvrAli) {
                if (exists($emailal{$r})) {
	            $info.=eval("\"".Locale::gettext::dgettext("ui", "email_cant_route_used")."\"") . "<BR>";
                    next;
                }
                push @goodal, $r;
            }
            $emailsvrali = join(',', @goodal);
            # tell sendmail that this address is local
            $ret = Cobalt::Email::mail_add_cw(@goodal);
            unless ($ret =~ /^2/) { 
                $info.=substr( $ret, 4 ).'<BR>';
	        $err=1;
            }
            # route email @server.alias.com --> @host.domain.com
            %emailal = map { $_ => $hostName } @goodal;
            $ret = Cobalt::Email::mail_virtuser_add_route(%emailal);
            unless ($ret =~ /^2/) { 
                $info.=substr( $ret, 4 ).'<BR>';
	        $err=1;
            }
 
        }
	
	# config cgi
	if( $cgi eq 'on' ) { $ret=vsite_set_cgis( $groupName, 1 ); }
	else { $ret=vsite_set_cgis( $groupName, 0 ); }
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
	
	# config ssi
	if( $ssi eq 'on' ) { $ret=vsite_set_ssi( $groupName, 1 ); }
	else { $ret=vsite_set_ssi( $groupName, 0 ); }
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
	
	# config apache's ssl
	vsite_set_sslconf($groupName, $ssl);

	# config fpx
	if($fpx eq 'on')
	{
	    $ret=fpx_set_web( $groupName, 1 );
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}
	
	# config bandwidth limits
	my ($bwchobj,$bwwarn);
	my $oldlimit = bw_getip($ipAddr);
	if ($bwlimit eq "on") {
	    foreach (@other_ipaddrs) {
		$bwchobj = Cobalt::Meta->create("type" => "vsite");
		$bwchobj->retrieve($_);
		if ($bwchobj->get("bwlimit") eq "off") {
		  Cobalt::Meta::update("type"   => "vsite",
				       "change" => ["bwlimit", "on"],
				       "where"  => ["name", "=", "$_"]);
		    $bwwarn=1;
		}
	    }
	    if ($oldlimit ne $bw_setip) {
		$ret = bw_setip("ip" => $ipAddr, "bwlimit" => $bw_setip, "protocol" => "tcp");
		if ($ret!~/^2/) {
		    $info.=substr ($ret, 4).'<BR>';
		    $err=1;
		}
		if (scalar(@other_ipaddrs)) {
		    $bwwarn=1;
		}
	    }

	    if ($bwwarn) {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "bw_setip_warn_conflict")."\"") . "<BR>";
		$err=1;
	    }

	} else {
	    # use existing default.
	    foreach (@other_ipaddrs) {
		$bwchobj = Cobalt::Meta->create("type" => "vsite");
		$bwchobj->retrieve($_);
		if ($bwchobj->get("bwlimit") eq "on") {
		    $bwlimit = 'on';
		    $bw_setip = $oldlimit;
		    $bwwarn=1;
		    last;
		}
	    }

	    if ($bwwarn) {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "bw_setip_warn_oldlimit")."\"") . "<BR>";
		$err=1;
	    }
	}

        Cobalt::Http::http_restart();
	
	my $default = Cobalt::Meta->create("type" => "vsite");
	$default->retrieve("default");
	
	my $def_user_quota = $default->get("user_quota");
	my $def_user_fpx = $default->get("user_fpx");
	my $def_user_shell = $default->get("user_shell");
	my $def_user_apop = $default->get("user_apop");
	my $def_user_namegen = $default->get("user_namegen");
	my $def_userlist_sort = $default->get("userlist_sort");
	my $def_userlist_range = $default->get("userlist_range");
	
	# save all vsite info in the database
	$obj->put("type" => "vsite",
		  "name" => $groupName,
		  "gid" => (getgrnam($groupName))[2],
		  "hostname" => $host,
		  "domain" => $domain,
		  "fqdn" => $hostName,
		  "ipaddr" => $ipAddr,
		  "fpx" => $fpx,
		  "quota" => $quota,
		  "maxusers" => $maxUsers,
		  "emaildomain" => $emailDomain,
		  "emailserveralias" => $emailsvrali,
		  "webdomain" => $webDomain,
		  "shell" => $shell,
		  "bwlimit" => $bwlimit,
		  "cgi" => $cgi,
		  "ssi" => $ssi,
		  "ssl" => $ssl,
		  "apop" => $apop,
		  "suspend" => $suspend,
		  "ftp" => $ftp,
		  "ftpusers" => $ftpUsers,
		  "ftpquota" => $fquota,
		  "user_quota" => $def_user_quota,
		  "user_fpx" => $def_user_fpx,
		  "user_shell" => $def_user_shell,
		  "user_apop" => $def_user_apop,
		  "user_namegen" => $def_user_namegen,
		  "userlist_sort" => $def_userlist_sort,
		  "userlist_range" => $def_userlist_range);
    }

    if($herr) {
	my $locstr = "error=yes";
	$locstr .= "&info=$info";
	$locstr .= "&hostname=$host";
	$locstr .= "&domain=$domain";
	$locstr .= "&ipaddr=$ipAddr";
	$locstr .= "&emaildomain=$emailDomain";
	$locstr .= "&webdomain=$webDomain";
	$locstr .= "&quota=$diskQuota";
	$locstr .= "&maxusers=$maxUsers";
	$locstr .= "&shell=$shell";
	$locstr .= "&cgi=$cgi";
	$locstr .= "&ssi=$ssi";
	$locstr .= "&ssl=$ssl";
	$locstr .= "&fpx=$fpx";
	$locstr .= "&ftp=$ftp";
	$locstr .= "&ftpusers=$ftpUsers";
	$locstr .= "&ftpquota=$ftpQuota";
	$locstr .= "&apop=$apop";
	$locstr .= "&bw_setip=$bw_setip";
	$locstr .= "&bwlimit=$bwlimit";
	print "location: $siteAddScriptURL\?$locstr\n\n";
    } elsif($err) {
	$obj->save;
	print "location: $siteListScriptURL\?info=$info\n\n";
    } else {
	$obj->save;
	print "location: $siteListScriptURL\n\n";
    }
}

#
# make a list of all the currently available disks.
#
sub makeVolList
{
    my %mounts = disk_getmounts($Vol_sitepattern);
    my (%maxfree, %list);
    my ($key, $text);

    foreach $key (keys %mounts) {
	# special case $Vol_home 
	if ($mounts{$key}{MountPoint} =~ /^\/$Vol_home/) {
	    $list{$Vol_home} = $mounts{$key}{Free};
	    if ($list{$Vol_home} > $maxfree{Value}) {
		$maxfree{Disk} = $Vol_home;
		$maxfree{Value} = $list{$Vol_home};
	    }
	    next;
	} 

	# everything else 
	$mounts{$key}{MountPoint} =~ /^$Vol_rootdir\/(\S+)$/o;
	next unless (-e "$mounts{$key}{MountPoint}/.cobaltsig"); 
	$list{$1} = $mounts{$key}{"Free"};
	if ($list{$1} > $maxfree{Value}) {
	    $maxfree{Disk} = $1;
	    $maxfree{Value} = $list{$1};
	}
    }

    $text = "<SELECT NAME=\"volume\">\n";
    foreach $key (sort keys %list) {
	$text .= "<OPTION VALUE=\"$key\" ";
	$text .= "SELECTED" if ($maxfree{Disk} eq $key);
	$text .= ">$key (" . int($list{$key} / 1024);
	$text .= " " . MSG_get_ui("volMBFree");
	$text .= ")</OPTION>\n";
    }
    $text .= "</SELECT>\n";

    return $text;
}

sub genSiteAddPage {
    my $obj = shift;
    my $bwobj = shift;
    my $info = shift;

    # find default CGI
    my $cgiChecked='';
    if ($obj->get("cgi") eq 'on') { $cgiChecked='CHECKED'; }

    # find default SSI
    my $ssiChecked='';
    if ($obj->get("ssi") eq 'on') { $ssiChecked='CHECKED'; }

    # find default SSL
    my $sslChecked='';
    if ($obj->get("ssl") eq 'on') { $sslChecked='CHECKED'; }

    # find default FPX
    my $fpxChecked='';
    if ($obj->get("fpx") eq 'on') { $fpxChecked='CHECKED'; }

    # find default shell account status
    my $shellChecked='';
    if ($obj->get("shell") eq 'on') { $shellChecked='CHECKED'; }

    # find default Domain Email
    my $emailDomainChecked='';
    if ($obj->get("emaildomain") eq 'on') { $emailDomainChecked='CHECKED'; }

    # find default Email Server Aliases
    my $emailSvrAli = $obj->get("emailserveralias");

    # find default web domain
    my $webDomainChecked='';
    if ($obj->get("webdomain") eq 'on') { $webDomainChecked='CHECKED'; }

    # find default APOP
    my $apopChecked='';
    if ($obj->get("apop") eq 'on') { $apopChecked='CHECKED'; }

    # find default BW
    my $bwChecked='';
    if ($obj->get("bwlimit") eq 'on') { $bwChecked='CHECKED'; }

    # find default FTP state
    my $ftpChecked='';
    if ($obj->get("ftp") eq 'on') { $ftpChecked='CHECKED'; }

    my $def_quota = $obj->get("quota");
    my $def_ftpquota = $obj->get("ftpquota");
    my $def_ftpusers = $obj->get("ftpusers");
    my $def_maxusers = $obj->get("maxusers");
    my $def_hostname = $obj->get("hostname");
    my $def_domain = $obj->get("domain");
    my $def_ipaddr = $obj->get("ipaddr");
    my $def_bwsetip = $bwobj->get("bwlimit");
    
    my $volList = makeVolList;

    # Read in the template
    open( TEMPLATE, "$Template_dir/siteAdd.tmpl" )
	|| die "Cannot read $Template_dir/siteAdd.tmpl: $!\n";

    print "content-type: text/html\n";
    print "Cache-Control: no-cache\n\n";

    while( <TEMPLATE> ) {
	s/\[USER_INFO\]/$info/o;
	s/\[CGICHECKED\]/$cgiChecked/o;
	s/\[SHELLCHECKED\]/$shellChecked/o;
	s/\[SSICHECKED\]/$ssiChecked/o;
	s/\[SSLCHECKED\]/$sslChecked/o;
	s/\[DEFAULTDISKQUOTA\]/$def_quota/o;
	s/\[FTPCHECKED\]/$ftpChecked/o;
	s/\[FTPQUOTA\]/$def_ftpquota/o;
	s/\[MAXFTPUSERS\]/$def_ftpusers/o;
	s/\[SITEADDSCRIPTURL\]/$siteAddScriptURL/o;
	s/\[SITELISTSCRIPTURL\]/$siteListScriptURL/o;
	s/\[EMAILDOMAINCHECKED\]/$emailDomainChecked/o;
	s/\[EMAILSERVERALIAS\]/$emailSvrAli/o;
	s/\[WEBDOMAINCHECKED\]/$webDomainChecked/o;
	s/\[MAXUSERS\]/$def_maxusers/o;
	s/\[APOPCHECKED\]/$apopChecked/o;
	s/\[FPXCHECKED\]/$fpxChecked/o;
	s/\[DOMAIN\]/$def_domain/o;
	s/\[HOSTNAME\]/$def_hostname/o;
	s/\[IPADDR\]/$def_ipaddr/o;
	s/\[BWLIMIT\]/$bwChecked/o;
	s/\[BW_SETIP\]/$def_bwsetip/o;
	s/\[VOLLISTCODE\]/$volList/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print $_;
    }
    close TEMPLATE;	    
}

