#!/usr/bin/perl

# author: Will DeHaan 
#         Cobalt Networks
# version: 1.0
# date: 01-30-99
#
# Copyright 1999, Cobalt Networks, Inc
# All rights reserved

# siteMod is intended to report site settings to site administrators.
# Site admins manage users, data, backups, that's it.  RaQ admins manage
# services and sites.  Thus, this page and other per-site service 
# configuration utilities (ftp) are only editable by the RaQ admin
# named $Adminuser.

# Import configuratoin utilities
use Cobalt::Product;
use Cobalt::BandWidth;
use Cobalt::Email;
use Cobalt::Fpx;
use Cobalt::Ftp;
use Cobalt::Group;
use Cobalt::List;
use Cobalt::Meta;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Network;
use Cobalt::Stats;
use Cobalt::User;
use Cobalt::Vsite;
use Cobalt::Http;
use Cobalt::Util;
use Cobalt::Locale;
use Cobalt::Layout;
use Cobalt::DiskUtils;

use DB_File;
use Fcntl qw(O_RDONLY O_RDWR O_CREAT);
use strict;
use CGI;

use Locale::gettext;
Locale::gettext::textdomain("ui");

# global variables
my $q=new CGI;
my $siteModScriptURL='/cgi-bin/.cobalt/siteMod/siteMod.cgi';
my $forbiddenURL="/.cobalt/error/forbidden.html";

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

# 'page' is a hidden field used to differentiate a completed form from 
# a form request
my $page=$q->param('page');
my $group=$q->param('group');

my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "group", $group)) {
    print "location: $forbiddenURL\n\n";    
    die "Unauthorised access by $user, not a member of $group.\n";
}

if ($page eq 'sitemod') {

    # read form input
    my $h_ipaddr=$q->param('ipaddr');
    my $h_hostname=$q->param('hostname');
    my $h_domain=$q->param('domain');
    my $h_fqdn=$h_hostname . '.' . $h_domain;
    my $h_cgi=$q->param('cgi');
    my $h_shell=$q->param('shell');
    my $h_ssi=$q->param('ssi');
    my $h_ssl=$q->param('ssl');
    my $h_fpx=$q->param('fpx');
    my $h_emaildomain=$q->param('emaildomain');
    my $h_emailsvrali=$q->param('emailserveralias');
    my $h_webdomain=$q->param('webdomain');
    my $h_quota=$q->param('diskquota');
    my $h_maxusers=$q->param('maxusers');
    my $h_bwlimit=$q->param('bwlimit');
    my $h_bw_setip=$q->param('bw_setip');
    my $h_apop=$q->param('apop');
    my $h_suspend=$q->param('suspend');

    # set nulls to "off" for database
    $h_fpx ? ($h_fpx = 'on') : ($h_fpx = 'off');
    $h_cgi ? ($h_cgi = 'on') : ($h_cgi = 'off');
    $h_ssi ? ($h_ssi = 'on') : ($h_ssi = 'off');
    $h_ssl ? ($h_ssl = 'on') : ($h_ssl = 'off');
    $h_apop ? ($h_apop = 'on') : ($h_apop = 'off');
    $h_suspend ? ($h_suspend = 'on') : ($h_suspend = 'off');
    $h_shell ? ($h_shell = 'on') : ($h_shell = 'off');
    $h_emaildomain ? ($h_emaildomain = 'on') : ($h_emaildomain = 'off');
    $h_webdomain ? ($h_webdomain = 'on') : ($h_webdomain = 'off');
    $h_bwlimit ? ($h_bwlimit = 'on') : ($h_bwlimit = 'off');

    my $hobj = Cobalt::Meta->new("type" => "vsite");
    $hobj->put("name" => $group,
	       "hostname" => $h_hostname,
	       "domain" => $h_domain,
	       "fqdn" => $h_fqdn,
	       "ipaddr" => $h_ipaddr,
	       "emaildomain" => $h_emaildomain,
	       "emailserveralias" => $h_emailsvrali,
	       "webdomain" => $h_webdomain,
	       "quota" => $h_quota,
	       "maxusers" => $h_maxusers,
	       "shell" => $h_shell,
	       "cgi" => $h_cgi,
	       "ssi" => $h_ssi,
	       "ssl" => $h_ssl,
	       "fpx" => $h_fpx,
	       "apop" => $h_apop,
	       "suspend" => $h_suspend,
	       "bwlimit" => $h_bwlimit);
    
    my $oldhobj = Cobalt::Meta->create("type" => "vsite");
    $oldhobj->retrieve($group);

    # bail with error unless user logged in
    unless ($ENV{'REMOTE_USER'}) {
	genSiteModPage($group, "Unauthenticated site update attempt");
    }

    # verify the input data
    my $result=verifySiteModPage($group, $hobj, $h_bw_setip, $oldhobj);

    # null returned if no error
    if ($result) {
	genSiteModPage($group,$result);
    } else {
	$result=handleSiteModPage($group, $hobj, $h_bw_setip, $oldhobj);
	genSiteModPage($group,$result);
    }

} else {
    genSiteModPage($group);
}

1;

sub verifySiteModPage
{
    my $group = shift;
    my $vobj = shift;
    my $bw_setip = shift;
    my $oldvobj = shift;

    my $ret;
    my $err=0;

    my $info='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';

    my $ipAddr=$vobj->get('ipaddr');
    my $hostName=$vobj->get('hostname');
    my $domain=$vobj->get('domain');
    my $cgi=$vobj->get('cgi');
    my $shell=$vobj->get('shell');
    my $ssi=$vobj->get('ssi');
    my $ssl=$vobj->get('ssl');
    my $fpx=$vobj->get('fpx');
    my $emailDomain=$vobj->get('emaildomain');
    my $webDomain=$vobj->get('webdomain');
    my $diskQuota=$vobj->get('quota');
    my $maxUsers=$vobj->get('maxusers');
    my $apop=$vobj->get('apop');
    my $suspend=$vobj->get('suspend');
    my $bwlimit=$vobj->get('bwlimit');

    # find out if another host has the same host.domain and or ipaddr w/ certain features on
    # find out if the same fqdn is used
    my $other_fqdns = scalar(Cobalt::Meta::query("type"  => "vsite",
						 "keys"  => ["name"],
						 "where" => ["name", "<>", "default",
							     "AND", "name", "<>", "$group",
							     "AND", "hostname", "=", "$hostName",
							     "AND", "domain", "=", "$domain"]));

    if ($other_fqdns) {
	my($name)=$hostName;
	$info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_hostnameinuse")."\"") . "<BR>";
	$err=1;
    }
    # find out if another site w/ same ip has ssl on
    if($ssl eq 'on') {
	if (scalar(Cobalt::Meta::query("type"  => "vsite",
				       "keys"  => ["name"],
				       "where" => ["name", "<>", "default",
						   "AND", "name", "<>", "$group",
						   "AND", "ipaddr", "=", "$ipAddr",
						   "AND", "ssl", "=", "on"]))) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_ipinuse_ssl")."\"") . "<BR>";
	    $err=1;
	}
    }
    # find out if another site with the same ip has emaildomain on
    if($emailDomain eq 'on') {
	if (scalar(Cobalt::Meta::query("type"  => "vsite",
				       "keys"  => ["name"],
				       "where" => ["name", "<>", "default",
						   "AND", "name", "<>", "$group",
						   "AND", "domain", "=", "$domain",
						   "AND", "emaildomain", "=", "on"]))) {
	    my $r = $domain;
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "email_cant_route_used")."\"") . "<BR>";
	    $err=1;
	}
    }
    # find out if another site with the same ip has webdomain on
    if($webDomain eq 'on') {
	if (scalar(Cobalt::Meta::query("type"  => "vsite",
				       "keys"  => ["name"],
				       "where" => ["name", "<>", "default",
						   "AND", "name", "<>", "$group",
						   "AND", "domain", "=", "$domain",
						   "AND", "webdomain", "=", "on"]))) {
	    my $r = $domain;
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_webdomain")."\"") . "<BR>";
	    $err=1;
	}
    }

    # Enforce the necessity of a maximum number of users for this site
    if ($maxUsers!~/^\d+$/o || $maxUsers==0)
    {
	$info.=MSG_get_ui("maxUsersNotValid") . "<BR>";
	$err=1;
    }

    my @siteusers = vsite_get_users($group);
    my($usercount)=scalar @siteusers;
    if ($maxUsers < $usercount) {
	$info.=MSG_get_ui("maxUsersLessthanCurrentUsers") . "<BR>";
	$err=1;
    }

    unless(netutil_test_domainname($hostName) && 
	   netutil_test_domainname($domain))
    {
	$info.=MSG_get_ui("hostNameRequired") . "<BR>";
	$err=1;
    }
    if( !$hostName || !$ipAddr || !$diskQuota ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_essential_data")."\"") . "<BR>";
	$err=1;
    }
    if( $hostName=~/localhost/io ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "localhostNotAllowed")."\"") . "<BR>";
	$err=1;
    }
    if( $hostName && !netutil_test_domainname( $hostName ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "hostNameNotValid")."\"") . "<BR>";
	$err=1;
    }
    if( !netutil_test_ipaddr( $ipAddr ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "addrNotValid")."\"") . "<BR>";
	$err=1;
    }
    if( $diskQuota!~/^\d+$/o || $diskQuota==0 ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotPositive")."\"") . "<BR>";
	$err=1;
    }
    if( $bw_setip < 10 && $bwlimit eq 'on' ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "bw_setip_invalid")."\"");
	$err=1;
    }

    if ($err) {
	return($info);
    } else {
	return;
    }
}

sub handleSiteModPage
{
    my $group = shift;
    my $vobj = shift;
    my $bw_setip = shift;
    my $oldvobj = shift;

    my $ret=0;
    my $err=0;
    my $info='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';

    my $ipAddr=$vobj->get('ipaddr');
    my $hostName=$vobj->get('hostname');
    my $domain=$vobj->get('domain');
    my $cgi=$vobj->get('cgi');
    my $shell=$vobj->get('shell');
    my $ssi=$vobj->get('ssi');
    my $ssl=$vobj->get('ssl');
    my $fpx=$vobj->get('fpx');
    my $emailDomain=$vobj->get('emaildomain');
    my $emailsvrali=$vobj->get('emailserveralias');
    my $webDomain=$vobj->get('webdomain');
    my $diskQuota=$vobj->get('quota');
    my $maxUsers=$vobj->get('maxusers');
    my $apop=$vobj->get('apop');
    my $suspend=$vobj->get('suspend');
    my $bwlimit=$vobj->get('bwlimit');

    # don't lose other things
    my $o_user_quota = $oldvobj->get("user_quota");
    my $o_user_fpx = $oldvobj->get("user_fpx");
    my $o_user_shell = $oldvobj->get("user_shell");
    my $o_user_apop = $oldvobj->get("user_apop");
    my $o_user_namegen = $oldvobj->get("user_namegen");
    my $o_userlist_sort = $oldvobj->get("userlist_sort");
    my $o_userlist_range = $oldvobj->get("userlist_range");

    # obtain old settings
    my $oldipAddr = $oldvobj->get("ipaddr");
    my $oldhostName = $oldvobj->get("hostname");
    my $olddomain = $oldvobj->get("domain");
    my $oldemail = $oldvobj->get("emaildomain");
    my $oldemailsvrali = $oldvobj->get("emailserveralias");
    my $oldweb = $oldvobj->get("webdomain");
    my $oldfpx = $oldvobj->get("fpx");
    my $oldftp = $oldvobj->get("ftp");
    my $oldcgi = $oldvobj->get("cgi");
    my $oldssi = $oldvobj->get("ssi");
    my $oldssl = $oldvobj->get("ssl");
    my $oldshell = $oldvobj->get("shell");
    my $oldbwlimit = $oldvobj->get("bwlimit");
    my $oldwebdomain = $oldvobj->get("webdomain");
    my $oldsuspend = $oldvobj->get("suspend");
    my $oldftpquota = $oldvobj->get("ftpquota");
    my $oldftpusers = $oldvobj->get("ftpusers");
    my $oldquota = $oldvobj->get("quota");

    $vobj->put("ftp"            => $oldftp,
	       "ftpusers"       => $oldftpusers,
	       "ftpquota"       => $oldftpquota,
	       "user_quota"     => $o_user_quota,
	       "user_fpx"       => $o_user_fpx,
	       "user_shell"     => $o_user_shell,
	       "user_apop"      => $o_user_apop,
	       "user_namegen"   => $o_user_namegen,
	       "userlist_sort"  => $o_userlist_sort,
	       "userlist_range" => $o_userlist_range);
    

    my $fqdn = $hostName.'.'.$domain;
    my $oldfqdn = $oldhostName.'.'.$olddomain;

    # config bandwidth limits
    #
    my (@other_newip, @other_oldip, $bwwarn);
    my $bwchobj = Cobalt::Meta->create("type" => "vsite");
    
    # if this is a name based site and they enabled bandwidth management or it is
    # already enabled for another virtual site, we need to warn them
    my @other_newip = Cobalt::Meta::query("type"  => "vsite",
					  "keys"  => ["name"],
					  "where" => ["name", "<>", "default",
						      "AND", "name", "<>", "$group",
						      "AND", "ipaddr", "=", "$ipAddr"]);
    my @other_oldip = Cobalt::Meta::query("type"  => "vsite",
					  "keys"  => ["name"],
					  "where" => ["name", "<>", "default",
						      "AND", "name", "<>", "$group",
						      "AND", "ipaddr", "=", "$oldipAddr"]);
    
    my $oldbw_setip = bw_getip($oldipAddr);
    my $newbw_setip = bw_getip($ipAddr);
    
    # Begin the upgrade process, concatonate errors, function order should
    # clear dependencies

    # Site will be re-added with new parameters immediately
    # viste_add/del manage virtual network interfaces so it is necessary
    # to formally delete the "old" site leaving site data intact and 
    # create its replacement
    if ($fqdn ne $oldfqdn || $ipAddr ne $oldipAddr) {
	# Fetch old FPX configuration, to be re-added if FPX remains enabled
	my @fpx_webs;
	if ($oldfpx eq 'on') {
	    @fpx_webs=vsite_get_fpx($group);

	    # deleting FrontPage has moved from vsite_del to site_del so we
	    # must call it here
            Cobalt::Fpx::fpx_set_web($group,0);
	}

	$ret = vsite_del($group);
	if ($ret!~/^2/o) {
	    $info.=substr( $ret, 4 ) . "<BR>";
	    $err=1;
	}
	
	# Set the default webmaster email address to the site group
	$ret = vsite_add($ipAddr,$fqdn,$group,$group);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ) . "<BR>";
	    $err=1;
	}

	if ($webDomain eq 'on') {
	    $ret = vsite_add_alias($group, $domain);
	    if ($ret!~/^2/) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	}

	my $fA = ($oldftp eq 'on') ? 1 : 0;
	$ret=ftp_config($ipAddr,1,$fA,$group,$oldftpusers,$oldftpquota);
	if ($ret!~/^2/o) {
	    $info.=substr ($ret, 4) . "<BR>";
	    # Non-critical error
	    $err=1;
	}

	# log file fragments in MB not blocks
	my $logRotateLimit=int($diskQuota/10);
	$logRotateLimit=1 unless ($logRotateLimit);
	# Rotate split log fragments for the site
	$ret=vsite_set_logrotate($group,$logRotateLimit);
	if ($ret!~/^2/) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	if ($fpx eq 'on') {
	    foreach my $fpxenable (@fpx_webs) {
                $ret = fpx_set_web($fpxenable, 1);
		if( $ret!~/^2/ ) {
		    $info.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
		# slowly update the UI with multiple calls to meta  
		if ($fpxenable ne $group) {
		    my $uobj = Cobalt::Meta->create("type" => "users");
		    $uobj->retrieve($fpxenable);
		    $uobj->put("fpx" => 'on');
		    $uobj->save;
		}
	    }
	}
    
	# things to do if the fqdn changed
	if ($fqdn ne $oldfqdn) {

	    # delete old cw entry
	    $ret=Cobalt::Email::mail_del_cw($oldfqdn);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }
	    # and add a new one
	    $ret=Cobalt::Email::mail_add_cw($fqdn);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }
	    # delete domain email route
	    $ret=Cobalt::Email::mail_virtuser_del_route($olddomain,$oldfqdn);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	    # delete domain email accept
	    $ret=Cobalt::Email::mail_del_cw($olddomain);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	    # delete domain relay
	    $ret=Cobalt::Email::mail_del_relay($olddomain);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }

	    if ($emailDomain eq 'on') {
		$ret=Cobalt::Email::mail_virtuser_add_route($domain,$fqdn);
		if( $ret!~/^2/o ) {
		    $info.=substr( $ret, 4 ) . "<BR>";
		    $err=1;
		}
		$ret=Cobalt::Email::mail_add_cw($domain);
		if( $ret!~/^2/o ) {
		    $info.=substr( $ret, 4 ) . "<BR>";
		    $err=1;
		}
		$ret=Cobalt::Email::mail_add_relay($domain);
		if ($ret!~/^2/o) {
		    $info.=substr($ret, 4) . "<BR>";
		    $err=1;
		}
	    }
	    
	    # delete the virtual list server
	    my @siteusers = vsite_get_users($group);
	    $ret=list_modify_virtual ($group, $oldfqdn, $fqdn);
	    if ($ret!~/^2/) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	    my %active = map {$_ => 1} @siteusers;
	    
	    my($siteUser,$alias); # for-loop instances 
	    my(%aliasDel,%aliasAdd);
	    my(%allAlias)=mail_virtuser_list_alias();
	    foreach $alias (keys %allAlias)
	    {
		if ($active{ $allAlias{$alias} })
		{
		    my($userAlias,$dom)=split(/\@/,$alias);
		    $aliasDel{$alias}=$allAlias{$alias};
		    $aliasAdd{ $userAlias.'@'.$fqdn }=$allAlias{$alias};
		} 
	    }
	    
	    # Delete old/pre-existing and default aliases, then (re)add
	    $ret=mail_virtuser_del_alias(%aliasDel);
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	    $ret=mail_virtuser_add_alias(%aliasAdd);
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	    $ret=mail_virtuser_dfl_aliases($oldfqdn, 0);
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
            }
            $ret=mail_virtuser_dfl_aliases($fqdn, 1);
            if( $ret!~/^2/ ) {
                $info.=substr( $ret, 4 ).'<BR>';
		$err=1;
            }
	    
	    # generate management site
	    # make layout file
	  Cobalt::Layout::layout_siteManage($group,$fqdn);

	} # end [fqdn changed]

	# Updates specific to only modified IP address
	if($oldipAddr ne $ipAddr) 
	{
	    # Allow relaying to modified IP
	    $ret = mail_del_relay($oldipAddr);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }
	    $ret = mail_add_relay($ipAddr);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }

	    # if we are moving from an ip and there are other name
	    # based sites, we want to warn about updating their configuration
	    if (!scalar(@other_oldip) && $oldbwlimit eq 'on') {
		$ret = bw_delip($oldipAddr);
		if ($ret!~/^2/) {
		    $info.=substr ($ret, 4).'<BR>';
		    $err=1;
		}
	    }

	    if ($bwlimit eq 'on') {
		foreach (@other_newip) {
		    $bwchobj->retrieve($_);
		    if ($bwchobj->get("bwlimit") eq "off") {
		      Cobalt::Meta::update("type"   => "vsite",
					   "change" => ["bwlimit", "on"],
					   "where"  => ["name", "=", "$_"]);
			$bwwarn=1;
		    }
		}
		if ($bw_setip ne $newbw_setip && scalar(@other_newip)) {
		    $bwwarn=1;
		}
		bw_setip("ip" => $ipAddr, "bwlimit" => $bw_setip);
		if ($ret!~/^2/) {
		    $info.=substr ($ret, 4).'<BR>';
		    $err=1;
		}
	    } else {
		foreach (@other_newip) {
		    $bwchobj->retrieve($_);
		    if ($bwchobj->get("bwlimit") eq "on") {
		      Cobalt::Meta::update("type"   => "vsite",
					   "change" => ["bwlimit", "off"],
					   "where"  => ["name", "=", "$_"]);
			$bwwarn=1;
		    }
		}
		if (scalar(@other_newip)) {
		    $bwwarn=1;
		}
		bw_delip($ipAddr);
		if ($ret!~/^2/) {
		    $info.=substr ($ret, 4).'<BR>';
		    $err=1;
		}
	    }
		    
	    if ($bwwarn) {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "bw_setip_warn_conflict")."\"") . "<BR>";
		$err=1;
	    }
	} # end [if ip changed]
    } # end [if fqdn or ip changed]
    else {

	# config logrotate
	if ($oldquota ne $diskQuota && $MaxQuota lt $diskQuota) {
	    # log file fragments in MB not blocks
	    my $logRotateLimit=int($diskQuota/10);
	    $logRotateLimit=1 unless ($logRotateLimit);
	    # Rotate split log fragments for the site
	    $ret=vsite_set_logrotate($group,$logRotateLimit);
	    if ($ret!~/^2/) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	# config bandwidth limits
	if ($oldbw_setip ne $bw_setip) {
	    $ret = bw_setip("ip" => $ipAddr, "bwlimit" => $bw_setip);
	    if ($ret!~/^2/) {
		$info.=substr ($ret, 4).'<BR>';
		$err=1;
	    }
	    if (scalar(@other_newip)) {
		$bwwarn=1;
	    }
	}
	if ($bwlimit eq "on" && $oldbwlimit eq "off") {
	    foreach (@other_newip) {
		$bwchobj->retrieve($_);
		if ($bwchobj->get("bwlimit") eq "off") {
		  Cobalt::Meta::update("type"   => "vsite",
				       "change" => ["bwlimit", "on"],
				       "where"  => ["name", "=", "$_"]);
		    $bwwarn=1;
		}
	    }
	} elsif ($bwlimit eq "off" && $oldbwlimit eq "on") {
	    foreach (@other_newip) {
		$bwchobj->retrieve($_);
		if ($bwchobj->get("bwlimit") eq "on") {
		  Cobalt::Meta::update("type"   => "vsite",
				       "change" => ["bwlimit", "off"],
				       "where"  => ["name", "=", "$_"]);
		    $bwwarn=1;
		}
	    }
	    $ret = bw_delip($ipAddr);
	    if ($ret!~/^2/) {
		$info.=substr ($ret, 4).'<BR>';
		$err=1;
	    }
	}
	if ($bwwarn) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "bw_setip_warn_conflict")."\"") . "<BR>";
	    $err=1;
	}

	# Delete the old fqdn and/or old domain from sendmail.cw
	# delete the accept-email-at-domain route if it should exist
	if ($oldemail eq 'on' && $emailDomain eq 'off') {
	    $ret=Cobalt::Email::mail_virtuser_del_route($domain,$fqdn);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	    $ret=Cobalt::Email::mail_del_cw($domain);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	    $ret=Cobalt::Email::mail_del_relay($domain);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }
	} elsif ($oldemail eq 'off' && $emailDomain eq 'on') {
	    $ret=Cobalt::Email::mail_virtuser_add_route($domain,$fqdn);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	    $ret=Cobalt::Email::mail_add_cw($domain);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	    $ret=Cobalt::Email::mail_add_relay($domain);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }
	}

	if ($webDomain eq 'on' && $oldwebdomain eq 'off') {
	    $ret = vsite_add_alias($group, $domain);
	    if ($ret!~/^2/) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	} elsif ($webDomain eq 'off' && $oldwebdomain eq 'on') {
	    $ret = vsite_del_alias($group, $domain);
	    if ($ret!~/^2/) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	}
    }

    # Revoke shell access if previously granted
    if ($oldshell eq 'on' && $shell eq 'off') {
	my(@siteUsers)=vsite_get_users($group);
	foreach (@siteUsers) 
	{
	    $ret=user_mod($_,'','','',$Badshell);
	    unless($ret=~/^2/)
	    {
		$info.=substr($ret,4).'<BR>';
		$err=1;
	    }
	}
    }

    # Grant shell access for users who had it previous to a site shell revoke
    if ($oldshell eq 'off' && $shell eq 'on') {
	my @enableUsers = Cobalt::Meta::query("type" => "users",
                                              "keys" => ["name"],
                                              "where" => ["shell", "=", "on"]);
        foreach (@enableUsers)
        {
            $ret=user_mod($_, '', '', '', $Usershell);
             unless($ret=~/^2/)
             {
                 $info.=substr($ret,4).'<BR>';
                 $err=1;
             }
        }
    }
		    
    # set disk quota
    $diskQuota=0 if ($diskQuota =~ /unlimited/i);
    $diskQuota*=1048576/$BytesPerBlock; # megabyte to block
    $ret=group_set_quota($group,$diskQuota);
    if ($ret!~/^2/o) {
	$info.=substr($ret, 4).'<BR>';
	$err=1;
    }

    # config cgi
    $ret=vsite_set_cgis($group , ($cgi eq 'on'));
    if ($ret!~/^2/) {
	$info.=substr ($ret, 4).'<BR>';
	$err=1;
    }

    # config ssi
    $ret=vsite_set_ssi($group , ($ssi eq 'on'));
    if ($ret!~/^2/) {
	$info.=substr ($ret, 4).'<BR>';
	$err=1;
    }

    # config ssl
    vsite_set_sslconf($group, $ssl);
    
    # config fpx
    if ($oldfpx eq 'off' && $fpx eq 'on') {
	$ret=fpx_set_web($group, 1);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    } elsif ($fpx eq 'off' && $oldfpx eq 'on') {
	$ret=fpx_set_web( $group, 0 );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    if (($oldsuspend eq 'off') && ($suspend eq 'on')) {
	$ret = vsite_disable($group);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    } elsif (($suspend eq 'off') && ($oldsuspend eq 'on')) {
	$ret = vsite_enable($group);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    # handle valid email server aliases
if ($emailsvrali ne $oldemailsvrali)  {
    my @emailSvrAli = split(',', $emailsvrali);
    my ($alias);
    foreach $alias( @emailSvrAli )  {
        if($alias eq $fqdn)  {
            $info.=eval("\"".Locale::gettext::dgettext("ui",
                "emailServerAliasInvalid")."\"") . "<BR>";
	    @emailSvrAli = ();
	    $emailsvrali = "";
            $vobj->put("emailserveralias" => $emailsvrali);
            $err=1;
        }
    }

    my @goodal = ();
    my %emailal = Cobalt::Email::mail_virtuser_list_route();
    if (scalar(@emailSvrAli)) {
        foreach my $r (@emailSvrAli) {
            if (exists($emailal{$r})) {
	        $info.=eval("\"".Locale::gettext::dgettext("ui", "email_cant_route_used")."\"") . "<BR>";
                next;
            }
            push @goodal, $r;
        }
        $emailsvrali = join(',', @goodal);
        # tell sendmail that this address is local
        $ret = Cobalt::Email::mail_add_cw(@goodal);
        unless ($ret =~ /^2/) { 
            $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
        }
        # route email @server.alias.com --> @host.domain.com
        %emailal = map { $_ => $hostName } @goodal;
        $ret = Cobalt::Email::mail_virtuser_add_route(%emailal);
        unless ($ret =~ /^2/) { 
            $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
        }
    }
}
	    
    Cobalt::Http::http_restart();
    Cobalt::Email::mail_sendmail_hup();
    sleep(5);

    # there was too many chances for a site to be almost added but
    # not get in the database, so i'd rather there be an entry in
    # the database but not have a complete site than have a bad
    # database so we save no matter what
    $vobj->save;

    if ($err) {
	return($info);
    } else {
	return;
    }
}

sub genSiteModPage
# Parses & prints the siteMod template
# Arguments: optional error text with image tags prepended
# Return value: true
# Site effects: none 
{
    my $group = shift;
    my $info = shift;

    my $vsite = Cobalt::Meta->create("type" => "vsite");
    $vsite->retrieve($group);
    
    my $diskQuota = $vsite->get("quota");
    my $ipAddr = $vsite->get("ipaddr");
    my $domainName = $vsite->get("domain");
    my $hostName = $vsite->get("hostname");
    my $apop = $vsite->get("apop");
    my $suspend = $vsite->get("suspend");
    my $shell = $vsite->get("shell");
    my $emailDomain = $vsite->get("emaildomain");
    my $emailsvrali = $vsite->get("emailserveralias");
    my $webDomain = $vsite->get("webdomain");
    my $maxUsers = $vsite->get("maxusers");
    my $bwlimit = $vsite->get("bwlimit");

    my $bwobj = Cobalt::Meta->create("type" => "bw");
    $bwobj->retrieve($ipAddr);
    my $bw_setip = $bwobj->get("bwlimit");

    # figure out the disk label.
    my $volume = disk_getlabel("$Htgroup_dir/$group");

    # $xRow consists of simple reported data or default-valued form fields
    # RaQ admin gets the editable form, site admin gets read-only data
    my ($cgiRow,$shellRow,$ssiRow,$sslRow,$fpxRow,$emailRow,$diskQuotaRow,$webdomainRow,$bwlimitRow);
    my ($maxUsersRow,$ipRow,$hostRow,$domainRow,$suspendRow,$bw_setip_row,$apopRow);

    if($user eq $Adminuser) 
    {
        # Disk quota set to "0" implies unlimited
	if($diskQuota==0) 
        {
	    $diskQuotaRow="<INPUT TYPE=TEXT NAME=\"diskquota\" VALUE=\"\" SIZE=\"5\">"; 
	} else
        {
	    $diskQuotaRow="<INPUT TYPE=TEXT NAME=\"diskquota\" VALUE=\"$diskQuota\" SIZE=\"5\" onChange=\"checkDiskQuota( this );\">";
        }

        # $xChecked represents either a default-valued checkbox or a 
        # yes/no image gif
        # Identify which services are enabled
 	my $cgiChecked='CHECKED' if ($vsite->get("cgi") eq 'on');
	my $ssiChecked='CHECKED' if ($vsite->get("ssi") eq 'on');
	my $sslChecked='CHECKED' if ($vsite->get("ssl") eq 'on');
        my $fpxChecked='CHECKED' if ($vsite->get("fpx") eq 'on');
	my $suspendChecked='CHECKED' if ($vsite->get("suspend") eq 'on');
        my $emailChecked='CHECKED' if ($vsite->get("emaildomain") eq 'on');
        my $webdomainChecked='CHECKED' if ($vsite->get("webdomain") eq 'on');
        my $bwlimitChecked='CHECKED' if ($vsite->get("bwlimit") eq 'on');
        my $shellChecked='CHECKED' if ($vsite->get("shell") eq 'on');
        my $apopChecked='CHECKED' if ($vsite->get("apop") eq 'on');

        # Generate html.. 
	$cgiRow="<INPUT TYPE=CHECKBOX NAME=\"cgi\" VALUE=\"on\" $cgiChecked>";
	$ssiRow="<INPUT TYPE=CHECKBOX NAME=\"ssi\" VALUE=\"on\" $ssiChecked>";
	$sslRow="<INPUT TYPE=CHECKBOX NAME=\"ssl\" VALUE=\"on\" $sslChecked>";
        $fpxRow="<INPUT TYPE=CHECKBOX NAME=\"fpx\" VALUE=\"on\" $fpxChecked>";
        $emailRow="<INPUT TYPE=CHECKBOX NAME=\"emaildomain\" VALUE=\"on\" $emailChecked>";
        $webdomainRow="<INPUT TYPE=CHECKBOX NAME=\"webdomain\" VALUE=\"on\" $webdomainChecked>";
        $bwlimitRow="<INPUT TYPE=CHECKBOX NAME=\"bwlimit\" VALUE=\"on\" $bwlimitChecked>";
	$maxUsersRow="<INPUT TYPE=TEXT NAME=\"maxusers\" VALUE=\"$maxUsers\" SIZE=\"5\" onChange=\"checkDiskQuota( this, maxUsersNotValid );\">";
	$bw_setip_row="<INPUT TYPE=TEXT NAME=\"bw_setip\" VALUE=\"$bw_setip\" SIZE=\"5\">";
        $shellRow="<INPUT TYPE=CHECKBOX NAME=\"shell\" VALUE=\"on\" $shellChecked>";
        $apopRow="<INPUT TYPE=CHECKBOX NAME=\"apop\" VALUE=\"on\" $apopChecked>";

        # Network settings for the main group are edited in the RaQ admin
        # interface, not in the site settings page.  This is bad ui design.
	if($group eq $MainGroup)
	{
	    # MainGroup settings changed through RaQ admin interface
	    $ipRow="<A HREF=\"/.cobalt/sysManage/index.html?goto=2,1\" TARGET=\"_top\">$ipAddr</A><INPUT TYPE=HIDDEN NAME=\"ipaddr\" VALUE=\"$ipAddr\"><INPUT TYPE=HIDDEN NAME=\"oldipaddr\" VALUE=\"$ipAddr\">";
	    $domainRow="<A HREF=\"/.cobalt/sysManage/index.html?goto=2,1\" TARGET=\"_top\">$domainName</A><INPUT TYPE=HIDDEN NAME=\"domain\" VALUE=\"$domainName\"><INPUT TYPE=HIDDEN NAME=\"olddomain\" VALUE=\"$domainName\">";
	    $hostRow="<A HREF=\"/.cobalt/sysManage/index.html?goto=2,1\" TARGET=\"_top\">$hostName</A><INPUT TYPE=\"HIDDEN\" NAME=\"hostname\" SIZE=\"24\" VALUE=\"$hostName\"><INPUT TYPE=\"HIDDEN\" NAME=\"oldhostname\" VALUE=\"$hostName\">";
            $suspendRow=MSG_get_ui("disabled");
	} else
	{
	    $ipRow="<INPUT TYPE=\"TEXT\" NAME=\"ipaddr\" SIZE=\"15\" VALUE=\"$ipAddr\" onChange=\"checkIPAddr( this, addrNotValid );\"><INPUT TYPE=\"HIDDEN\" NAME=\"oldipaddr\" VALUE=\"$ipAddr\">";
	    $hostRow="<INPUT TYPE=\"TEXT\" NAME=\"hostname\" SIZE=\"24\" VALUE=\"$hostName\" onChange=\"checkDomainName( this, hostNameNotValid );\"><INPUT TYPE=\"HIDDEN\" NAME=\"oldhostname\" VALUE=\"$hostName\">";
	    $domainRow="<INPUT TYPE=\"TEXT\" NAME=\"domain\" SIZE=\"24\" VALUE=\"$domainName\" onChange=\"checkDomainName( this, domainNameNotValid1 );\"><INPUT TYPE=\"HIDDEN\" NAME=\"olddomain\" VALUE=\"$domainName\">";
	    $suspendRow="<INPUT TYPE=CHECKBOX NAME=\"suspend\" VALUE=\"on\" $suspendChecked>";
	} 

    } else # Site admin, do not allow editable fields, note the yes/no gifs
           # instead of checkboxes
    {
	$diskQuotaRow = $diskQuota;
	$maxUsersRow=$maxUsers;
	$bw_setip_row=$bw_setip;
	$ipRow=$ipAddr;
	$hostRow=$hostName;
	$domainRow=$domainName;

	if($vsite->get("cgi") eq 'on')
        { 
            $cgiRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{
            $cgiRow=MSG_get_ui("disabled");
        }

	if($vsite->get("ssi") eq 'on')
        { 
            $ssiRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{ 
            $ssiRow=MSG_get_ui("disabled");
        }

	if($vsite->get("ssl") eq 'on')
        { 
            $sslRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{ 
            $sslRow=MSG_get_ui("disabled");
        }
	
	if($vsite->get("fpx") eq 'on')
        { 
            $fpxRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{ 
            $fpxRow=MSG_get_ui("disabled");
        }

	if($vsite->get("suspend") eq 'on') 
        { 
            $suspendRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{ 
            $suspendRow=MSG_get_ui("disabled");
        }

	if($vsite->get("emaildomain") eq 'on') 
        { 
            $emailRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{ 
            $emailRow=MSG_get_ui("disabled");
        }
	
	if($vsite->get("webdomain") eq 'on') 
        { 
            $webdomainRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{ 
            $webdomainRow=MSG_get_ui("disabled");
        }
	
	if($vsite->get("bwlimit") eq 'on') 
        { 
            $bw_setip_row=$bw_setip . " " . MSG_get_ui("kbps");
	    $bwlimitRow='';
        } else
	{ 
            $bw_setip_row=MSG_get_ui("disabled");
	    $bwlimitRow='';
        }
	
	if($vsite->get("shell") eq 'on') 
        { 
            $shellRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{ 
            $shellRow=MSG_get_ui("disabled");
        }

	if($vsite->get("apop") eq 'on') 
        { 
            $apopRow="<IMG SRC=\"/.cobalt/images/yes.gif\">"; 
        } else
	{ 
            $apopRow=MSG_get_ui("disabled");
        }
    }

    # figure out quota string
    my $quotamsg = MSG_get_ui("volSiteQuota",$volume);
    $diskQuotaRow .= " $quotamsg";

    # Html headers..
    print "content-type: text/html\nCache-Control: no-cache\n\n";
    
    # Read in the template
    if($ENV{'REMOTE_USER'} eq 'admin')
    {
        open( TEMPLATE, "$Template_dir/siteMod.tmpl" )
    	    || die "Cannot read $Template_dir/siteMod.tmpl: $!\n";
    } else
    {
        open( TEMPLATE, "$Template_dir/showSite.tmpl" )
    	    || die "Cannot read $Template_dir/showSite.tmpl: $!\n";
    }

    while( <TEMPLATE> ) {
	s/\[USER_INFO\]/$info/;
	s/\[CGI\]/$cgiRow/;
	s/\[SSI\]/$ssiRow/;
	s/\[SSL\]/$sslRow/;
	s/\[SHELL\]/$shellRow/;
	s/\[FPX\]/$fpxRow/;
	s/\[SUSPEND\]/$suspendRow/;
	s/\[MAXUSERS\]/$maxUsersRow/;
	s/\[BWLIMIT\]/$bwlimitRow/;
	s/\[BW_SETIP\]/$bw_setip_row/;
	s/\[APOP\]/$apopRow/;
	s/\[EMAILDOMAIN\]/$emailRow/;
	s/\[EMAILSERVERALIAS\]/$emailsvrali/;
	s/\[WEBDOMAIN\]/$webdomainRow/;
	s/\[DISKQUOTA\]/$diskQuotaRow/;
	s/\[GROUP\]/$group/;
	s/\[HOSTNAME\]/$hostRow/;
	s/\[IPADDR\]/$ipRow/;
	s/\[DOMAIN\]/$domainRow/;
	s/\[SITEMODSCRIPTURL\]/$siteModScriptURL/;
	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/geo;

	print;
    }
    close TEMPLATE;
}
