#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 2.0
# date: 7-6-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Product;
use Cobalt::Email;
use Cobalt::List;
use Cobalt::Locale;
use Cobalt::User;
use Cobalt::Vacation;
use Cobalt::Fpx;
use Cobalt::Http;
use Cobalt::Meta;

# use strict;
use CGI;

# global variables
my $q=new CGI;

my $userName=$q->param('name');
my $group=$q->param('group');

my $siteUserDelScriptURL='/cgi-bin/.cobalt/siteUserDel/siteUserDel.cgi';
my $siteUserListScriptPath="$AdminCGIs_dir/siteUserList/siteUserList.cgi";
my $forbiddenURL="/.cobalt/error/forbidden.html";

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }


# force login
my $loginName=$ENV{ REMOTE_USER };

if( !$loginName ) {
    # ask user to enter user name
    print "location: $siteUserDelScriptURL?username=$userName&group=$group\n\n";
} elsif (not user_authok($loginName, "group", $group)) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorised access by $loginName, not a member of $group.\n";
} else {
    handleSiteUserDelPage();
}

1;


#
# event handler for delete user page
#
sub handleSiteUserDelPage {
    my $err=0;
    my $info='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';

    # Remove Fpx user web
    # This must be run before user_del since fpx_get_web checks user existance
    if ($Features{'fpx'}) {
        if (fpx_get_web($userName)) {
            fpx_set_web($userName, 0);
	}
	# return value is ignored, a failed fpx user web deletion is not 
	# critical to system functionality
    }

    my $ret=user_del( $userName );
    if( $ret!~/^2/ ) {
	$info.=substr( $ret, 4 ).'<BR>';
	$err=1;
    }

    if( !$err ) {
	# remove email forward address if necessary
        if( alias_exist( $userName ) ) {
            $ret=alias_delete( $userName );
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	# remove email aliases
	$ret=mail_virtuser_set_byuser( $userName );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# turn off vacation
	$ret=vacation_set_on( $userName, 0 );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
	# no vacation message
	$ret=vacation_set_message( $userName, '' );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    # remove the user from the database
    my $uobj=Cobalt::Meta->create("type" => "users");
    $uobj->retrieve($userName);
    $uobj->delete;

    # remove the user from the defcon table if needed
    if (scalar(Cobalt::Meta::query("type" => "defcon",
                                   "keys" => ["name"],
                                   "where" => ["name", "=", "$userName"]))) {
 
        my $dobj=Cobalt::Meta->create("type" => "defcon");
        $dobj->retrieve($userName);
        $dobj->delete;
    }                      


    if( $err ) {
	# generate error message
	$ENV{ PARAM_GROUP }=$group;
	$ENV{ PARAM_INFO }=$info;
	require $siteUserListScriptPath;
	delete $ENV{ PARAM_GROUP };
	delete $ENV{ PARAM_INFO };
    }
    else {
	$ENV{ PARAM_GROUP }=$group;
	require $siteUserListScriptPath;
	delete $ENV{ PARAM_GROUP };
    }

    Cobalt::Http::http_restart();
}
