#!/usr/bin/perl

# author: Kevin K.M. Chiu
#         Will DeHaan
# version: 2.0
# date: 7-8-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Product;
use Cobalt::Admserv;
use Cobalt::Email;
use Cobalt::Locale;
use Cobalt::Layout;
use Cobalt::Netutil;
use Cobalt::Network;
use Cobalt::Quota;
use Cobalt::Time;
use Cobalt::User;
use Cobalt::Services;
use Cobalt::Util;
use Cobalt::swatch;

use CGI;
use strict;
use Locale::gettext;

# global variables
my $q=new CGI;

my $sysSettingsScriptURL='/cgi-bin/.cobalt/sysSettings/sysSettings.cgi';
my $registerScriptURL='/cgi-bin/.cobalt/register/register.cgi';
my $wizardScript="$AdminCGIs_dir/wizard/wizardDone.cgi";

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $page=$q->param( 'page' );

if( $page eq 'syssettings' ) { 
    handleSysSettingsPage(\*STDOUT, $q);
} elsif( $page eq 'syssettingsconfirm' ) { 
    handleSysSettingsConfirmPage(\*STDOUT, $q); 
} else {
    genSysSettingsPage( \*STDOUT );
}
1;


sub handleSysSettingsPage {
    # network parameters
    my ($handle, $q) = @_;
    my $hostName=$q->param( 'hostname' );
    my $domainName=$q->param( 'domainname' );
    my $dnsAddr1=$q->param( 'dnsaddr1' );
    my $dnsAddr2=$q->param( 'dnsaddr2' );
    my $ipAddr=$q->param( 'ipaddr' );
    my $netMask=$q->param( 'netmask' );
    my $gateway=$q->param( 'gateway' );
    my $emailOn=$q->param( 'email' );
    my $dnsOn=$q->param( 'dns' );

    # admin parameters
    my $password1=$q->param( 'password1' );
    my $password2=$q->param( 'password2' );

    # time parameters
    my $month=$q->param( 'month' );
    my $date=$q->param( 'date' );
    my $year=$q->param( 'year' );
    my $hour=$q->param( 'hour' );
    my $minute=$q->param( 'minute' );
    my $timeZone=$q->param( 'zonelist' );
    my $region = $q->param('regionlist');
    my $country = $q->param('countrylist');
#    my $ntpServer=$q->param( 'ntpserver' );

    # check password
    my @passwordInBytes=unpack( 'C*', $password1 );
    my $passwordBadChar=0;
    my $byte;
    foreach $byte ( @passwordInBytes ) {
	if( $byte>127 ) { $passwordBadChar=1; last; }
    }

    my $result='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';
    my $err=0;
    my $ret;

    # error check for network settings
    if( !$hostName ) {
	$result.= MSG_get_ui("noHostName");
	$err=1;
    } elsif ( $hostName=~/localhost/io ) {
	$result.= MSG_get_ui("localhostNotAllowed");
	$err=1;
    } elsif( $hostName && !netutil_test_domainname( $hostName ) ) {
	$result.= MSG_get_ui("hostNameNotValid");
	$err=1;
    } elsif( $domainName && !netutil_test_domainname( $domainName ) ) {
	$result.= MSG_get_ui("domainNameNotValid");
	$err=1;
    } elsif( !$ipAddr ) {
	$result.= MSG_get_ui("noIPAddr");
	$err=1;
    } elsif( !netutil_test_ipaddr( $ipAddr ) ) {
	$result.= MSG_get_ui("addrNotValid");
	$err=1;
    } elsif( !$netMask ) {
	$result.= MSG_get_ui("noNetMask");
	$err=1;
    } elsif( !netutil_test_ipaddr( $netMask ) ) {
	$result.= MSG_get_ui("maskNotValid");
	$err=1;
    } elsif( $dnsAddr1 && !netutil_test_ipaddr( $dnsAddr1 ) ) {
	my $dnsAddr=$dnsAddr1;
	$result.= MSG_get_ui("dnsNotValid");
	$err=1;
    } elsif( $dnsAddr2 && !netutil_test_ipaddr( $dnsAddr2 ) ) {
	my $dnsAddr=$dnsAddr2;
	$result.= MSG_get_ui("dnsNotValid");
	$err=1;
    } elsif( $gateway && !netutil_test_ipaddr( $gateway ) ) {
	$result.= MSG_get_ui("gatewayNotValid");
	$err=1;
    }

    # error check for administration settings
    elsif( !$password1 || !$password2 ) {
	$result.= MSG_get_ui("noPasswords");
	$err=1;
    } elsif( $password1 ne $password2 ) {
	$result.= MSG_get_ui("passwordsNotMatch");
	$err=1;
    } elsif( $passwordBadChar ) {
	$result.= MSG_get_ui("passwordBadChar");
	$err=1;
    } else {
	# everything is fine

	#
	# apply time settings
	#
	my ($oldzone, $oldcountry, $oldregion) = time_get_zone;
	if (($timeZone ne $oldzone) or ($region ne $oldregion) or
	    ($country ne $oldcountry)) {
	    $ret=time_set_zone( $timeZone, $country, $region);
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	# set time
	my ($oldmonth, $oldday, $oldyear, $oldhour, $oldminute) = 
	    ($q->param('oldmonth'), $q->param('oldday'), 
	     $q->param('oldyear'), $q->param('oldhour'),
	     $q->param('oldminute'));

	if (($oldyear ne $year) or ($oldmonth ne $month) or 
	    ($oldday ne $date) or ($oldhour ne $hour) or 
	    ($oldminute ne $minute)) {
	    $ret=set_time( $month, $date, $year, $hour, $minute, 1);
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	#
	# apply administration settings
	#
	my $ret=user_mod( $Adminuser, Locale::gettext::dgettext("special-sauce", "DefaultAdminFullName"), $password1, 0 );
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	#
	# apply network settings
	#
	# change IP address?
	my $ipConfirm=0;
	my $oldIPAddr=( net_get_ipaddr() )[ 0 ];
	if( ( $oldIPAddr ne $ipAddr ) ) { $ipConfirm=1; }

	# check dns servers and gateway
	my $dns1Confirm=0;
	my $dns2Confirm=0;
	my $gatewayConfirm=0;
	my ( $oldDNSAddr1, $oldDNSAddr2 )=( net_get_dnsclient() )[ 1, 2 ];

	# Pop-up confirmation to use unavailable DNS server
	# setting IP to self ($ipAddr) is ok as DNS is turned on locally

	# Primary DNS server
	if ( $dnsAddr1 && 
           ( $dnsAddr1 ne $oldDNSAddr1 ) && 
           ( $dnsAddr1 ne $ipAddr ) &&
           netutil_test_dns_server( $dnsAddr1 )!~/^2/o ) { 
           $dns1Confirm=1;
        }

	# Optional secondary DNS server
	if ( $dnsAddr2 && 
           ( $dnsAddr2 ne $oldDNSAddr2 ) && 
           ( $dnsAddr2 ne $ipAddr ) &&
            netutil_test_dns_server( $dnsAddr2 )!~/^2/o ) { 
          $dns2Confirm=1;
        }

	if( defined($gateway) and ( $gateway ne ( net_get_gateway() )[ 0 ] ) 
            and netutil_ping( $gateway )!~/^2/o ) { 
	    $gatewayConfirm=1; 
        }

	# Update services
        my $ftpOn=$q->param( 'ftp' );
        if( $ftpOn=~/on/o ) {
	    Cobalt::swatch::swatch_set_service_on("ftp");
	    $ret=Cobalt::Services::set_ftp_server_on( 1 );
	} else {
	    Cobalt::swatch::swatch_set_service_off("ftp");
	    $ret=Cobalt::Services::set_ftp_server_on( 0 );
	}
        if( $ret!~/^2/o ) {
            $result.=substr( $ret, 4 ).'<BR>';
            $err=1;
        }

        my $telnetOn=$q->param( 'telnet' );
        if( $telnetOn=~/on/o ) {
	    Cobalt::swatch::swatch_set_service_on("telnet");
	    $ret=set_telnet_server_on( 1 );
	} else {
	    Cobalt::swatch::swatch_set_service_off("telnet");
	    $ret=set_telnet_server_on( 0 );
	}
        if( $ret!~/^2/o ) {
            $result.=substr( $ret, 4 ).'<BR>';
            $err=1;
        }

	if( $Features{ 'smb' } ) {
	    my $smbOn=$q->param( 'smb' );
	    if( $smbOn=~/on/o ) { $ret=Cobalt::Services::set_smb_server_on( 1 ); }
	    else { $ret=Cobalt::Services::set_smb_server_on( 0 ); }
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	if( $Features{ 'atalk' } ) {
	    my $appleShareOn=$q->param( 'appleshare' );
	    if( $appleShareOn=~/on/o ) { $ret=Cobalt::Services::set_atalk_server_on( 1 ); }
	    else { $ret=Cobalt::Services::set_atalk_server_on( 0 ); }
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	if( $Features{ 'snmp' } ) {
	    my $snmpOn=$q->param( 'snmp' );
	    if( $snmpOn=~/on/o ) {
	        Cobalt::swatch::swatch_set_service_on("snmp");
		$ret=Cobalt::Services::set_snmp_server_on( 1 );
	    } else {
	        Cobalt::swatch::swatch_set_service_off("snmp");
		$ret=Cobalt::Services::set_snmp_server_on( 0 );
	    }
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	if( $Features{ 'networker' } ) {
	    my $networkerOn=$q->param( 'networker' );
	    if( $networkerOn=~/on/o ) { $ret=Cobalt::Services::set_networker_client_on( 1 ); }
	    else { $ret=Cobalt::Services::set_networker_client_on( 0 ); }
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	if( $Features{ 'dhcpd' } ) {
	    my $dhcpOn=$q->param( 'dhcp' );
	    if( $dhcpOn=~/on/o ) { $ret=Cobalt::Services::set_dhcp_server_on( 1 ); }
	    else { $ret=Cobalt::Services::set_dhcp_server_on( 0 ); }
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	if( $Features{ 'ntp' } ) {
	    my $ntpOn=$q->param( 'ntp' );
	    if( $ntpOn=~/on/o ) { $ret=Cobalt::Services::set_ntp_server_on( 1 ); }
	    else { $ret=Cobalt::Services::set_ntp_server_on( 0 ); }
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	# if no confirmation needed
	if( $dns1Confirm || $dns2Confirm || $gatewayConfirm || $ipConfirm ) {
	    return genConfirmPage( $handle, $hostName, $domainName, $dnsAddr1, $dnsAddr2, $ipAddr, $netMask, $gateway, $dns1Confirm, $dns2Confirm, $ipConfirm, $gatewayConfirm , $emailOn, $dnsOn );
	} else {
	    $ret=net_set_ipaddr( $ipAddr, $netMask );
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }

	    if( $gateway ) {
		$ret=net_set_gateway( $gateway );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
	    }

	    $ret=net_set_dnsclient( $domainName, $dnsAddr1, $dnsAddr2 );
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }

	    if ($Features{'dns'})
	    {
                # force the DNS server on if self-assigned
		$dnsOn='on' if ( ($dnsAddr1 eq $ipAddr) || 
				 ($dnsAddr2 eq $ipAddr) );
            	if ( $dnsOn=~/on/o ) 
                { 
		    Cobalt::swatch::swatch_set_service_on("dns");
		    $ret=Cobalt::Services::set_dns_server_on( 1 ); 
		}
            	else 
		{ 
		    Cobalt::swatch::swatch_set_service_off("dns");
		    $ret=Cobalt::Services::set_dns_server_on( 0 ); 
		}
            	if( $ret!~/^2/o ) {
                	$result.=substr( $ret, 4 ).'<BR>';
                	$err=1;
            	}
            }

	    if( !$err ) {
		$ret=net_set_hostname( $hostName );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}

	        $ret=mail_add_cw( "$hostName.$domainName");
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
	    }

	    # generate site management site
	    genMainSiteManage( netutil_get_fulldomain() );

   	    # Configure email after DNS server available
            if( $emailOn=~/on/o ) { 
	      Cobalt::Services::set_mail_server_on( 1 ); 
	      Cobalt::swatch::swatch_set_service_on("smtp");
	      Cobalt::swatch::swatch_set_service_on("pop3");
	      Cobalt::swatch::swatch_set_service_on("imap");
	    } else { 
	      Cobalt::Services::set_mail_server_on( 0 ); 
	      Cobalt::swatch::swatch_set_service_off("smtp");
	      Cobalt::swatch::swatch_set_service_off("pop3");
	      Cobalt::swatch::swatch_set_service_off("imap");
	    }

	    return wizardDone($handle, $ipAddr);
	}
    }

    if( $err ) {
	genSysSettingsPage( $handle, $result );
    }
}

sub handleSysSettingsConfirmPage {
    # read form inputs
    my ($handle, $q) = @_;
    my $hostName=$q->param( 'hostname' );
    my $domainName=$q->param( 'domainname' );
    my $dnsAddr1=$q->param( 'dnsaddr1' );
    my $dnsAddr2=$q->param( 'dnsaddr2' );
    my $ipAddr=$q->param( 'ipaddr' );
    my $netMask=$q->param( 'netmask' );
    my $gateway=$q->param( 'gateway' );
    my $emailOn=$q->param( 'email' );
    my $dnsOn=$q->param( 'dns' );

    my ( $oldDomainName, $oldDNSAddr1, $oldDNSAddr2 )=net_get_dnsclient();
    if( !$dnsAddr1 ) { $dnsAddr1=$oldDNSAddr1; }
    if( !$dnsAddr2 ) { $dnsAddr2=$oldDNSAddr2; }

    if( !$gateway ) { $gateway=( net_get_gateway() )[ 0 ]; }

    my ( $oldIPAddr, $oldNetMask )=net_get_ipaddr();
    if( !$ipAddr ) { $ipAddr=$oldIPAddr; }
    if( !$netMask ) { $netMask=$oldNetMask; }

    my $result='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';
    my $err=0;
    my $ret;

    # set configuration
    if( $gateway ) {
	$ret=net_set_gateway( $gateway );
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    $ret=net_set_dnsclient( $domainName, $dnsAddr1, $dnsAddr2 );
    if( $ret!~/^2/o ) {
	$result.=substr( $ret, 4 ).'<BR>';
	$err=1;
    }

    if( !$err ) {
	$ret=net_set_hostname( $hostName );
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

        $ret=mail_add_cw($hostName,$domainName);
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    if( !$err ) {
	# do this last
	$ret=net_set_ipaddr( $ipAddr, $netMask );
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    if (!$err) {
	# generate site management site
	genMainSiteManage( netutil_get_fulldomain() );

	if ($Features{'dns'})
	{
            # force the DNS server on if self-assigned
	    $dnsOn='on' if ( ($dnsAddr1 eq $ipAddr) || 
	                     ($dnsAddr2 eq $ipAddr) );
            if ( $dnsOn=~/on/o ) 
            { 
		Cobalt::swatch::swatch_set_service_on("dns");
	        $ret=Cobalt::Services::set_dns_server_on( 1 ); 
	    }
            else 
	    { 
		Cobalt::swatch::swatch_set_service_off("dns");
	        $ret=Cobalt::Services::set_dns_server_on( 0 ); 
	    }
            if( $ret!~/^2/o ) {
               	$result.=substr( $ret, 4 ).'<BR>';
                $err=1;
            }
        }

	wizardDone($handle, $ipAddr);

   	# Configure email after DNS server available
        if( $emailOn=~/on/o ) {
	    Cobalt::swatch::swatch_set_service_on("smtp");
	    Cobalt::swatch::swatch_set_service_on("pop3");
	    Cobalt::swatch::swatch_set_service_on("imap");
	    Cobalt::Services::set_mail_server_on( 1 );
	} else {
	    Cobalt::swatch::swatch_set_service_off("smtp");
	    Cobalt::swatch::swatch_set_service_off("pop3");
	    Cobalt::swatch::swatch_set_service_off("imap");
	    Cobalt::Services::set_mail_server_on( 0 );
	}

	if( $ipAddr && ( $ipAddr ne $oldIPAddr ) ) {
	    system( '/sbin/shutdown -r now' );
	}
    } else {
	genSysSettingsPage($handle, $result );
    }
}

#
# argument 0 is host name
# argument 1 is domain name
# argument 2 is primary DNS
# argument 3 is secondary DNS
# argument 4 is IP address
# argument 5 is network mask
# argument 6 is gateway
# argument 7 is true if primary DNS confirmation is needed, false otherwise
# argument 8 is true if secondary DNS confirmation is needed, false otherwise
# argument 9 is true if IP address will be changed, false otherwise
# argument 10 is true if gateway confirmation is needed, false otherwise
#
sub genConfirmPage {
    my ( $handle, $hostName, $domainName, $dnsAddr1, 
	 $dnsAddr2, $ipAddr, $netMask, $gateway, 
	 $dnsConfirm1, $dnsConfirm2, $ipConfirm, 
	 $gatewayConfirm, $emailOn, $dnsOn ) = @_;

    my $dnsConfirmCode1;
    if( $dnsConfirm1 ) {
	my $dnsAddr=$dnsAddr1;
	$dnsConfirmCode1="if( !confirm( '".eval("\"".Locale::gettext::dgettext("ui", "dnsChangeConfirm")."\"")."' ) ) document.forms[ 0 ].dnsaddr1.value='';";
    }

    my $dnsConfirmCode2;
    if( $dnsConfirm2 ) {
	my $dnsAddr=$dnsAddr2;
	$dnsConfirmCode2="if( !confirm( '".eval("\"".Locale::gettext::dgettext("ui", "dnsChangeConfirm")."\"")."' ) ) document.forms[ 0 ].dnsaddr2.value='';";
    }

    my $gatewayConfirmCode;
    if( $gatewayConfirm ) {
	$gatewayConfirmCode="if( !confirm( '".eval("\"".Locale::gettext::dgettext("ui", "gatewayChangeConfirm")."\"")."' ) ) document.forms[ 0 ].gateway.value='';";
    }

    my $ipAlertCode;
    if( $ipConfirm ) {
	$ipAlertCode="alert( '".eval("\"".Locale::gettext::dgettext("ui", "ipChangeAlert")."\"")."' ); top.showInfo();";
    }

    print $handle <<END;
content-type: text/html
Cache-Control: no-cache

<HTML>
<BODY BGCOLOR=#FFFFFF onLoad="interact()">

<FORM METHOD="POST" ACTION="$sysSettingsScriptURL">
<INPUT TYPE=HIDDEN NAME="hostname" VALUE="$hostName">
<INPUT TYPE=HIDDEN NAME="domainname" VALUE="$domainName">
<INPUT TYPE=HIDDEN NAME="dnsaddr1" VALUE="$dnsAddr1">
<INPUT TYPE=HIDDEN NAME="dnsaddr2" VALUE="$dnsAddr2">
<INPUT TYPE=HIDDEN NAME="ipaddr" VALUE="$ipAddr">
<INPUT TYPE=HIDDEN NAME="netmask" VALUE="$netMask">
<INPUT TYPE=HIDDEN NAME="gateway" VALUE="$gateway">
<INPUT TYPE=HIDDEN NAME="email" VALUE="$emailOn">
<INPUT TYPE=HIDDEN NAME="dns" VALUE="$dnsOn">

<INPUT TYPE=HIDDEN NAME="page" VALUE="syssettingsconfirm">
</FORM>

<SCRIPT LANGUAGE="javascript">
<!-- hide
function interact() {
$dnsConfirmCode1
$dnsConfirmCode2
$gatewayConfirmCode
$ipAlertCode
document.forms[ 0 ].submit();
}
// hide end -->
</SCRIPT>
</BODY>
</HTML>
END
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the information shown to the user
#
sub genSysSettingsPage {
    my ( $handle, $info )=@_;
    
    #
    # get network information
    #
    my $hostName=net_get_hostname();
    if( $hostName=~/localhost/io ) { $hostName=''; }
    my ( $domainName, $dnsAddr1, $dnsAddr2 )=net_get_dnsclient();
    my ( $ipAddr, $netMask, $mac )=net_get_ipaddr();
    my $gateway=( net_get_gateway() )[ 0 ];

    # 
    # get services status
    #
    my $emailChecked='CHECKED'; # Always default to on

    my $ftpChecked=Cobalt::Services::get_ftp_server_on()?'CHECKED':'';
    my $dnsChecked=Cobalt::Services::get_dns_server_on()?'CHECKED':'';
    my $telnetChecked=get_telnet_server_on()?'CHECKED':'';
    my $smbChecked=Cobalt::Services::get_smb_server_on()?'CHECKED':'';
    my $appleShareChecked=Cobalt::Services::get_atalk_server_on()?'CHECKED':'';
    my $snmpChecked=Cobalt::Services::get_snmp_server_on()?'CHECKED':'';
    my $networkerChecked=Cobalt::Services::get_networker_client_on()?'CHECKED':'';
    my $dhcpChecked=Cobalt::Services::get_dhcp_server_on()?'CHECKED':'';
    my $ntpChecked=Cobalt::Services::get_ntp_server_on()?'CHECKED':'';

    #
    # get time information
    #
    my @time=localtime();
    my ($timeZone, $country, $region) = time_get_zone;
    my ($oldyear, $oldmonth, $oldday, $oldhour, $oldminute);

    my $i;
    my $monthsSel;
    for ( $i=0; $i < 12; $i++) {
	my $month = MSG_get_ui("month" . ($i + 1));

        if( $i==$time[ 4 ]) {
            $monthsSel .= "<OPTION VALUE=\"$i\" SELECTED>$month";
            $oldmonth = $i;
        } else {
            $monthsSel .= "<OPTION VALUE=\"$i\">$month";
        }
    }

    my $daysSel;
    for( $i=1; $i<=31; $i++ ) {
	if( $i==$time[ 3 ] ) { 
	    $daysSel .= "<OPTION VALUE=\"$i\" SELECTED>$i"; 
	    $oldday = $i;
	} else { 
	    $daysSel .= "<OPTION VALUE=\"$i\">$i"; 
	}
    }

    my $yearsSel;
    for( $i=99; $i<=138; $i++ ) {
	my $year=$i+1900;
	if( $i==$time[ 5 ] ) { 
	    $yearsSel .= "<OPTION VALUE=\"$year\" SELECTED>".$year; 
	    $oldyear = $year;
	} else { 
	    $yearsSel .= "<OPTION VALUE=\"$year\">".$year; 
	}
    }

    my $hoursSel;
    my $am=MSG_get_ui("ante_meridian");
    my $pm=MSG_get_ui("post_meridian");
    for( $i=0; $i<=23; $i++ ) {
	my $h=$i%12;
	if( $h==0 ) { $h=12; }
	if( $i<12 ) { $h.=" $am"; }
	else { $h.=" $pm"; }

	if( $i==$time[ 2 ] ) { 
	    $hoursSel .= "<OPTION VALUE=\"$i\" SELECTED>$h"; 
	    $oldhour = $i;
	} else { 
	    $hoursSel .= "<OPTION VALUE=\"$i\">$h"; 
	}
    }

    my $minutesSel;
    for( $i=0; $i<=59; $i++ ) {
	my $digit=$i;
	if( $digit>=0 && $digit<=9 ) { $digit='0'.$digit; }
	if( $i==$time[ 1 ] ) { 
	    $minutesSel .= "<OPTION VALUE=\"$i\" SELECTED>$digit"; 
	    $oldminute = $i;
	} else { 
	    $minutesSel .= "<OPTION VALUE=\"$i\">$digit"; 
	}
    }

#    my $ntpServer=( ntpc_get() )[ 1 ];

    # display locale specific date format
    my $dateFormat=$locale->getProperty( 'dateFormat' );
    my $hideMMDDYYStart='';
    my $hideMMDDYYEnd='';
    my $hideDDMMYYStart='';
    my $hideDDMMYYEnd='';
    my $hideYYMMDDStart='';
    my $hideYYMMDDEnd='';
    if( $dateFormat eq 'MMDDYY' ) {
	$hideDDMMYYStart='<!-- ';
	$hideDDMMYYEnd=' -->';
	$hideYYMMDDStart='<!-- ';
	$hideYYMMDDEnd=' -->';
    }
    elsif( $dateFormat eq 'DDMMYY' ) {
	$hideMMDDYYStart='<!-- ';
	$hideMMDDYYEnd=' -->';
	$hideYYMMDDStart='<!-- ';
	$hideYYMMDDEnd=' -->';
    }
    elsif( $dateFormat eq 'YYMMDD' ) {
	$hideMMDDYYStart='<!-- ';
	$hideMMDDYYEnd=' -->';
	$hideDDMMYYStart='<!-- ';
	$hideDDMMYYEnd=' -->';
    }

    # timezone information
    my $javascript = time_zone_js;

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    # Read in the template
    open( TEMPLATE, "$Template_dir/sysSettings.tmpl" )
	|| die "Cannot read $Template_dir/sysSettings.tmpl: $!\n";

    while ( <TEMPLATE> ) {
	# network
	s/\[HOSTNAME\]/$hostName/o;
	s/\[DOMAINNAME\]/$domainName/o;
	s/\[DNSADDR1\]/$dnsAddr1/o;
	s/\[DNSADDR2\]/$dnsAddr2/o;
	s/\[IPADDR\]/$ipAddr/go;
	s/\[NETMASK\]/$netMask/go;
	s/\[GATEWAY\]/$gateway/go;
	s/\[MAC\]/$mac/o;

	# administration
	s/\[ADMINUSER\]/$Adminuser/o;

        # services
        s/\[EMAILCHECKED\]/$emailChecked/o;
        s/\[FTPCHECKED\]/$ftpChecked/o;
        s/\[TELNETCHECKED\]/$telnetChecked/o;

	if( $Features{ 'discuss' } ) {
	    s/\[HIDEDISCUSSSTART\]//o;
	    s/\[HIDEDISCUSSEND\]//o;
	}
	else {
	    s/\[HIDEDISCUSSSTART\]/<!-- /o;
	    s/\[HIDEDISCUSSEND\]/ -->/o;
	}

	if( $Features{ 'smb' } ) {
	    s/\[SMBCHECKED\]/$smbChecked/o;
	    s/\[HIDESMBSTART\]//o;
	    s/\[HIDESMBEND\]//o;
	}
	else {
	    s/\[HIDESMBSTART\]/<!-- /o;
	    s/\[HIDESMBEND\]/ -->/o;
	}

	if( $Features{ 'atalk' } ) {
	    s/\[APPLESHARECHECKED\]/$appleShareChecked/o;
	    s/\[HIDEATALKSTART\]//o;
	    s/\[HIDEATALKEND\]//o;
	}
	else {
	    s/\[HIDEATALKSTART\]/<!-- /o;
	    s/\[HIDEATALKEND\]/ -->/o;
	}

	if( $Features{ 'snmp' } ) {
	    s/\[SNMPCHECKED\]/$snmpChecked/o;
	    s/\[HIDESNMPSTART\]//o;
	    s/\[HIDESNMPEND\]//o;
	}
	else {
	    s/\[HIDESNMPSTART\]/<!-- /o;
	    s/\[HIDESNMPEND\]/ -->/o;
	}

	if( $Features{ 'networker' } ) {
	    s/\[NETWORKERCHECKED\]/$networkerChecked/o;
	    s/\[HIDENETWORKERSTART\]//o;
	    s/\[HIDENETWORKEREND\]//o;
	}
	else {
	    s/\[HIDENETWORKERSTART\]/<!-- /o;
	    s/\[HIDENETWORKEREND\]/ -->/o;
	}

	if( $Features{ 'dns' } ) {
	    s/\[DNSCHECKED\]/$dnsChecked/o;
	    s/\[HIDEDNSSTART\]//o;
	    s/\[HIDEDNSEND\]//o;
	}
	else {
	    s/\[HIDEDNSSTART\]/<!-- /o;
	    s/\[HIDEDNSEND\]/ -->/o;
	}

	if( $Features{ 'dhcpd' } ) {
	    s/\[DHCPCHECKED\]/$dhcpChecked/o;
	    s/\[HIDEDHCPDSTART\]//o;
	    s/\[HIDEDHCPDEND\]//o;
	}
	else {
	    s/\[HIDEDHCPDSTART\]/<!-- /o;
	    s/\[HIDEDHCPDEND\]/ -->/o;
	}

	if( $Features{ 'ntp' } ) {
	    s/\[NTPCHECKED\]/$ntpChecked/o;
	    s/\[HIDENTPSTART\]//o;
	    s/\[HIDENTPEND\]//o;
	}
	else {
	    s/\[HIDENTPSTART\]/<!-- /o;
	    s/\[HIDENTPEND\]/ -->/o;
	}

	# time
	s/\[DAYS\]/$daysSel/o;
	s/\[HIDEDDMMYYEND\]/$hideDDMMYYEnd/o;
	s/\[HIDEDDMMYYSTART\]/$hideDDMMYYStart/o;
	s/\[HIDEMMDDYYEND\]/$hideMMDDYYEnd/o;
	s/\[HIDEMMDDYYSTART\]/$hideMMDDYYStart/o;
	s/\[HIDEYYMMDDEND\]/$hideYYMMDDEnd/o;
	s/\[HIDEYYMMDDSTART\]/$hideYYMMDDStart/o;
	s/\[HOURS\]/$hoursSel/o;
	s/\[MINUTES\]/$minutesSel/o;
	s/\[MONTHS\]/$monthsSel/o;
#	s/\[NTPSERVER\]/$ntpServer/o;
	s/\[YEARS\]/$yearsSel/o;
	s/\[TIMEJAVASCRIPT\]/$javascript/o;
	s/\[OLDHOUR\]/$oldhour/o;
	s/\[OLDMINUTE\]/$oldminute/o;
	s/\[OLDDAY\]/$oldday/o;
	s/\[OLDMONTH\]/$oldmonth/o;
	s/\[OLDYEAR]/$oldyear/o;
	s/\[TIMEZONE]/$timeZone/o;
	s/\[REGION]/$region/o;
	s/\[COUNTRY]/$country/o;
    

	s/\[USER_INFO\]/$info/o;
	s/\[SCRIPTURL\]/$sysSettingsScriptURL/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print $handle $_;
    }

    close TEMPLATE;
}


#
# argument 0 is the fully qualified domain
#
sub genMainSiteManage {
    my $fqdn = shift;

    layout_siteManage($MainGroup, $fqdn);

    # access control
    admserv_set_access( "$AdminPages_dir/siteManage/$MainGroup", 'require', 'group', $MainGroup );
}

sub wizardDone
{
	my ($handle, $ip) = @_;
        
	# limit quota for group users to 10 MB
	my $quota = 10*1048576/$BytesPerBlock;
	setqlim($Htgroup_dir, (getgrnam("users"))[2], $quota, $quota + 1, 1);

	# more work dumped here!! setup cron/custodiat and start 'em up.
	symlink($Cron_script, $Cron_link);
	system("$Cron_script start >/dev/null 2>&1");

	# Create default pages
	system("$wizardScript > /dev/null 2>&1\n");

	# send off the location for the registration wizard.
        print $handle "location: $registerScriptURL\n\n";

        # Restart admserv to update the hostname for rewrites
        my($pid,$l);
        open(PID,$AdminPid) || exit 1;
        $l = <PID>;
        $pid = $1 if ($l =~ /^(\d+)$/o); # Stricter than chomp
        close PID;
        kill('USR1',$pid) if ($pid);
}

