#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 2.0
# date: 7-4-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Product;
use Cobalt::Language;
use Cobalt::Locale;
use Cobalt::Time;
use Cobalt::Util;
use Locale::gettext;

use strict;
use CGI;

my $q=new CGI;

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $timeHTMLScriptURL='/cgi-bin/.cobalt/time/timehtml.cgi';
my $securityPageURL='/.cobalt/security/indexw.html?reload';

my $page=$q->param( 'page' );

if( $page ne 'time' ) { 
    genTimePage();
} else {
    handleTimePage($q);
}
1;


sub handleTimePage {
    my $q = shift;
    my $result='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';
    my $err=0;
    my ($ret, $manual_set);

    # always sync against the ntp server if it's there.
    my $ntpServer=$q->param( 'ntpserver' );
    $ret = $ntpServer ? ntpc_set(1, $ntpServer) : ntpc_set(0);
    if( $ret!~/^2/o ) {
	$result.=substr( $ret, 4 ).'<BR>';
	$err=1;
    }

    # timezone stuff.
    my ($oldzone, $oldcountry, $oldregion) = time_get_zone;
    my $zone = $q->param('zonelist');
    my $region = $q->param('regionlist');
    my $country = $q->param('countrylist');
    if (($oldzone ne $zone) or ($oldcountry ne $country) or
	($oldregion ne $region)) {
	$ret=time_set_zone($zone, $country, $region);
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    # set time
    my $oldyear = $q->param('oldyear');
    my $oldmonth = $q->param('oldmonth');
    my $oldday = $q->param('oldday');
    my $oldhour = $q->param('oldhour');
    my $oldminute = $q->param('oldminute');
    my $year = $q->param('year');
    my $month = $q->param('month');
    my $day = $q->param('date');
    my $hour = $q->param('hour');
    my $minute = $q->param('minute');
    
    if (($oldyear ne $year) or ($oldmonth ne $month) or 
	($oldday ne $day) or ($oldhour ne $hour) or
	($oldminute ne $minute)) {
	$manual_set = 1;
	$err = 0 if $ntpServer;
	$ret=set_time( $month, $day, $year, $hour, $minute);
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    if ( $err ) { 
	genTimePage( $result ); 
    } elsif ( $q->param( 'wizard' ) ) { 
	print "location: $securityPageURL\n\n"; 
#    } elsif ($manual_set) {
#      # this is here to work around a problem with time and apache
#      open( TEMPLATE, "$Template_dir/timeupdate.tmpl" )
#          || die "Cannot read $Template_dir/timeupdate.tmpl: $!\n";
#
#      print "content-type: text/html \nCache-Control: no-cache\n\n";
#      while( <TEMPLATE> ) {
#          s/\[TIMESCRIPTURL\]/$timeHTMLScriptURL/o;
#          s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;
#          print;
#      }
    } else {
	genTimePage();
    }
}

#
# argument 0 is the information shown to user
#
sub genTimePage {
    my $info = shift;

    my $field;
    my ($yearsSel,$monthsSel,$daysSel,$hoursSel,$minutesSel);
    my ($oldyear, $oldmonth, $oldday, $oldhour, $oldminute);

    my ($zone, $country, $region) = time_get_zone;
    my @time=localtime;
    my $t;

    my $i;
    for( $i=0; $i<12; $i++ ) {
	my $month=eval("\"".Locale::gettext::dgettext("ui", "month".($i+1))."\"");
	if( $i==$time[ 4 ] ) { 
	    $monthsSel .= "<OPTION VALUE=\"$i\" SELECTED>$month"; 
	    $oldmonth = $i;
	} else { 
	    $monthsSel .= "<OPTION VALUE=\"$i\">$month"; 
	}
    }

    for( $i=1; $i<=31; $i++ ) {
	if( $i==$time[ 3 ] ) { 
	    $daysSel .= "<OPTION VALUE=\"$i\" SELECTED>$i"; 
	    $oldday = $i;
	} else { 
	    $daysSel .= "<OPTION VALUE=\"$i\">$i"; 
	}
    }

    for( $i=99; $i<=138; $i++ ) {
	my $year=$i+1900;
	if( $i==$time[ 5 ] ) { 
	    $yearsSel .= "<OPTION VALUE=\"$year\" SELECTED>".$year; 
	    $oldyear = $year;
	} else { 
	    $yearsSel .= "<OPTION VALUE=\"$year\">".$year; 
	}
    }

    my $am = MSG_get_ui('ante_meridian');
    my $pm = MSG_get_ui('post_meridian');
    for( $i=0; $i<=23; $i++ ) {
	my $h=$i%12;
	if( $h==0 ) { $h=12; }
	if( $i<12 ) { $h.= " $am"; }
	else { $h.= " $pm"; }

	if( $i==$time[ 2 ] ) { 
	    $hoursSel .= "<OPTION VALUE=\"$i\" SELECTED>$h"; 
	    $oldhour = $i;
	} else { 
	    $hoursSel .= "<OPTION VALUE=\"$i\">$h"; 
	}
    }

    for( $i=0; $i<=59; $i++ ) {
	my $digit=$i;
	if( $digit>=0 && $digit<=9 ) { $digit='0'.$digit; }
	if( $i==$time[ 1 ] ) { 
	    $minutesSel .= "<OPTION VALUE=\"$i\" SELECTED>$digit"; 
	    $oldminute = $i;
	} else { 
	    $minutesSel .= "<OPTION VALUE=\"$i\">$digit"; 
	}
    }

    my $ntpServer=( ntpc_get() )[ 1 ];

    if( $q->param( 'wizard' ) eq 't' ) {
        $field='<INPUT TYPE=HIDDEN NAME="wizard" VALUE="t">';
    }
    else {
	my $save_string = MSG_get_ui("saveChanges");
	$field=<<END;
<A HREF="javascript: if( trySubmit() ) document.forms[ 0 ].submit();"
onMouseOver="status='$save_string'; return true;">
<IMG SRC="/.cobalt/images/save_but.gif" BORDER=0></A>
END
    }

    # display locale specific date format
    my $dateFormat=$locale->getProperty( 'dateFormat' );
    my $hideMMDDYYStart='';
    my $hideMMDDYYEnd='';
    my $hideDDMMYYStart='';
    my $hideDDMMYYEnd='';
    my $hideYYMMDDStart='';
    my $hideYYMMDDEnd='';
    if( $dateFormat eq 'MMDDYY' ) {
	$hideDDMMYYStart='<!-- ';
	$hideDDMMYYEnd=' -->';
	$hideYYMMDDStart='<!-- ';
	$hideYYMMDDEnd=' -->';
    }
    elsif( $dateFormat eq 'DDMMYY' ) {
	$hideMMDDYYStart='<!-- ';
	$hideMMDDYYEnd=' -->';
	$hideYYMMDDStart='<!-- ';
	$hideYYMMDDEnd=' -->';
    }
    elsif( $dateFormat eq 'YYMMDD' ) {
	$hideMMDDYYStart='<!-- ';
	$hideMMDDYYEnd=' -->';
	$hideDDMMYYStart='<!-- ';
	$hideDDMMYYEnd=' -->';
    }

    my $javascript = time_zone_js;

    # Read in the template
    open( TEMPLATE, "$Template_dir/timehtml.tmpl" )
	|| die "Cannot read $Template_dir/timehtml.tmpl: $!\n";

    print "content-type: text/html \nCache-Control: no-cache\n\n";

    while( <TEMPLATE> ) {
	s/\[TIME_INFO\]/$info/o;
	s/\[DAYS\]/$daysSel/o;
	s/\[HOURS\]/$hoursSel/o;
	s/\[HIDEDDMMYYEND\]/$hideDDMMYYEnd/o;
	s/\[HIDEDDMMYYSTART\]/$hideDDMMYYStart/o;
	s/\[HIDEMMDDYYEND\]/$hideMMDDYYEnd/o;
	s/\[HIDEMMDDYYSTART\]/$hideMMDDYYStart/o;
	s/\[HIDEYYMMDDEND\]/$hideYYMMDDEnd/o;
	s/\[HIDEYYMMDDSTART\]/$hideYYMMDDStart/o;
	s/\[MINUTES\]/$minutesSel/o;
	s/\[MONTHS\]/$monthsSel/o;
	s/\[NTPSERVER\]/$ntpServer/o;
	s/\[TIMEHTMLSCRIPTURL\]/$timeHTMLScriptURL/o;
	s/\[USER_INFO\]/$_[ 0 ]/o;
	s/\[YEARS\]/$yearsSel/o;
	s/\[OLDMINUTE\]/$oldminute/o;
	s/\[OLDHOUR\]/$oldhour/o;
	s/\[OLDDAY\]/$oldday/o;
	s/\[OLDMONTH\]/$oldmonth/o;
	s/\[OLDYEAR\]/$oldyear/o;
	s/\[TIMEZONE\]/$zone/o;
	s/\[COUNTRY\]/$country/o;
	s/\[REGION\]/$region/o;
	s/\[JAVASCRIPTDATA\]/$javascript/o;
	s/\[INPUT_FIELD\]/$field/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print;
    }
    close TEMPLATE;
}
