#!/usr/bin/perl

# Copyright Cobalt Networks 1999
#
# Author: Harris Vaegan-Lloyd
#
# Create an html page and graph of usage form a tab seperated file.
#

#use strict;
use GIFgraph::bars;
use CGI;

use Cobalt::Vsite;
use Cobalt::Product;
use Cobalt::Util;
use Cobalt::Locale;

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $scriptURL = "/cgi-bin/.cobalt/totUsage/totUsage.cgi";
my @colours = qw(dred dgreen dblue dyellow dbrown dorange dgray dpurple 
                 dpink gray blue green red yellow brown orange purple pink 
                 lgray gold lyellow lred lpurple lorange cyan );

my $q = new CGI;

sub tab_to_array {
	my $tab = shift;
	my @return;
	while (<$tab>) {
		chomp;
		my $line = $_;
		push (@return, [split(/\t/, $line)] );
	}
	return @return;			# returns a LOL
}

open (TAB, "$Report_dir/stats.tab") ||
	die "Could not not stats.tab $!\n";

my @stats = tab_to_array(\*TAB);

if ( $#stats < 0 || ! $stats[0][0] ) {
#if ( $#stats == 0 || ! $stats[0][0] ) {
	# If there is nothing to report on.
	print $q->header();
	open ( TEMPLATE, "$Template_dir/webUsageNoReport.tmpl" ) ||
		die "Cannot read $Template_dir/totUsage.tmpl: $!";

	while (<TEMPLATE>) {
		s/\[str.([^\]]+)\]/&MSG_get_ui("$1")/geo;
		print $_;
	}
	exit 0;
}

close TAB;

my $graph = new GIFgraph::bars;
my @graph_data;
#my @legend;

$graph->set(
	title		=> '',
	x_labels_vertical => 1, # Left align.
	dclrs		=> \@colours,
	y_number_format => \&make_y_label,
	overwrite 	=> $q->url_param('stack') # can be used to see th total overall..
);

my ($dates, $start_month, $start_year, $months_count) = get_period(@stats);
$graph_data[0] = $dates;

my $i = 0;
my @reports = $q->param("report_for");
unless ( $reports[0] ) {
	@reports = ("Total web", "Total mail", "Total ftp" );
}

foreach ( @reports ) {
	my @temp= split(/ /, $_);
	$reports[$i] = \@temp;
	$i++;
}

my $colour_tab = "<CENTER>\n<TABLE BORDER=0 WIDTH=120>\n";
$i = 0;

foreach my $report ( @reports ) {
	push @graph_data, get_data($$report[0], $$report[1], $start_year, $start_month, $months_count, @stats);
	my $string;
	if ( $$report[1] eq 'mail' ) {
		$string = &MSG_get_ui('mailUsageSummary');
	} elsif ($$report[1] eq 'ftp') {
		$string = &MSG_get_ui('ftpUsageSummary');
	} elsif ($$report[1] eq 'web') {
		$string = &MSG_get_ui('webUsageSummary');
	}
	$string = &MSG_get_ui("$$report[0]") . ' ' . $string;

	my $colour = sprintf('#%02x%02x%02x',GIFgraph::colour::_rgb($colours[$i]));
		
	$colour_tab .= <<END;
		<TR>
		    <TD WIDTH=20>
			<TABLE CELLSPACING=0 BORDER=0 BGCOLOR="$colour" WIDTH=15>
			    <TR><TD WIDTH=15><FONT COLOR="$colour"> &nbsp; </FONT></TD></TR>
			</TABLE>
		    </TD>
		    <TD WIDTH=100> $string </TD>
                </TR>
END
    ;
	# e.g. 'Total main usage', 'www.site.com ftp usage'
	unless ($colours[$i++]) {
		$i = 0;
	}
}
$colour_tab .= "</TABLE>\n</CENTER>\n";

#$graph->set_legend( @legend );

if ($q->url_param('image')) {
	print $q->header(-type => 'image/gif', -expires=> 'now' );
	print $graph->plot( \@graph_data ) ||
		die "Could not plot graph: $!\m";
	exit 0;
} else {
	print $q->header();
	my $query_line = '?image=yes&stack=' . $q->param('stack');
	foreach ( @reports ) {
		$query_line .= '&report_for='.$$_[0].'%20'.$$_[1];
	}

	my @selections;

	my %reports;
	my %labels;
	foreach (@stats)  {
	        # $reports{'Total mail'} = 1;
	        $reports{"$$_[0] $$_[1]"} = 1;
		$labels{"$$_[0] $$_[1]"} = &MSG_get_ui("$$_[0]") . ' ' . 
		    &MSG_get_ui("$$_[1]"."UsageSummary");
	}
	
	@selections = keys %reports;
	@selections = sort @selections;

	my $list = "\n\n<SELECT NAME=\"report_for\" SIZE=5 MULTIPLE>";
	foreach (@selections) {
	    $list .= "\n<OPTION VALUE=\"$_\">\n";
	    $list .= $labels{$_};
	    $list .= "\n</OPTION>";
	}
	$list .= "\n</SELECT>\n\n";

	# Returns our own URl..
	# We need to remove the query part of the string from the url..
	
	# make the bar placement toggles. we don't do this in a loop
	# because the labels are not orderly.
	my $bar_order;
	$bar_order .= "\n<INPUT TYPE=RADIO NAME=stack";
	$bar_order .= " CHECKED" unless $q->param('stack');
	$bar_order .= " VALUE=0>\n";
	$bar_order .= &MSG_get_ui("dontStack");

	$bar_order .= "\n&nbsp;&nbsp; <INPUT TYPE=RADIO NAME=stack";
	$bar_order .= " CHECKED" if $q->param('stack') eq 2;
	$bar_order .= " VALUE=2>\n";
	$bar_order .= &MSG_get_ui("stackOnTop");

	$bar_order .= "\n&nbsp;&nbsp; <INPUT TYPE=RADIO NAME=stack ";
	$bar_order .= " CHECKED" if $q->param('stack') eq 1;
	$bar_order .= " VALUE=1>\n";
	$bar_order .= &MSG_get_ui("stackInFront");
	$bar_order .= "\n";

	open ( TEMPLATE, "$Template_dir/totUsage.tmpl" ) ||
		die "Cannot read $Template_dir/totUsage.tmpl: $!";
	while (<TEMPLATE>) {
		s/\[SCRIPTURL\]/$scriptURL/o;
		s/\[IMGPARAMS\]/$query_line/o;;
		s/\[REPORTS\]/$list/o;
		s/\[LEGEND\]/$colour_tab/o;
		s/\[BARORDER\]/$bar_order/o;
		s/\[str.([^\]]+)\]/&MSG_get_ui("$1")/geo;
		print;
	}
	close (TEMPLATE);
}

1;

sub get_period {
	my @stats = @_;
	my $start_year = 9999;
	my $end_year = 0;
	my $start_month = $stats[0][3];
	my $end_month = 0;
	foreach (@stats) {
		if ( $$_[2] < $start_year ) {
			$start_year = $$_[2];
			$start_month = $$_[3];
		}
		if ( $$_[2] > $end_year ) {
			$end_year = $$_[2];
			$end_month = $$_[3];
		}
		$start_month = $$_[3] if $$_[3] < $start_month &&
				$start_year == $$_[2];
		$end_month = $$_[3] if $$_[3] > $end_month &&
				$end_year == $$_[2];
	}

	my @dates;
	my $months_count;

	foreach my $year ( $start_year .. $end_year ) {
		foreach my $month ( 1 .. 12 ) {
			next if ($year == $start_year && $month < $start_month);
			next if ($year == $end_year && $month > $end_month);
			push @dates, "$year/". &MSG_get_ui("month$month");
			$months_count++;
		}
	}
    # We should have at least a count of one thing to report so that
    # GIFgraph doesn't barf on 0 element arrays..
    # Additionally it will mean that we have an empty area. We should
    # Have some defaults..
    if (! $months_count ) {
        $start_month = $start_year = $months_count = 1;
        $dates[0] = '';
	}
	return \@dates, $start_month, $start_year, $months_count;
}
		

sub get_data {
	my $site = shift;
	my $area = shift;
	my $start_year = shift;
	my $start_month = shift;
	my $count = shift;
	my @info = @_;
	my @data;

	# As it stand it has the number of elements of the array, we want it to
	# have the index of thie highest number in the array..
	$count--;
	for ( 0 .. $count) {
		$data[$_] = 0;
	}
	
	foreach (@info) {
		if ($$_[0] eq $site && $$_[1] eq $area) {
			# Find out where we have to put it....
			my $posistion = ($$_[2] - $start_year) * 12 +
							($$_[3] - $start_month);
			$data[$posistion] = $$_[5];
		}
	}
	return \@data;
}


sub make_y_label
{
	my $number = shift;

	if ( $number < 1048576 ) {
		return int($number/1024) . 'KB';
	} elsif ( $number < 1073741824 ) {
		return int($number/1048576) . 'MB';
	# Wow.. Busy RaQ this one.. Over a gig of traffic..
	} else {
		return int($number/1073741824) . 'GB';
	}
}
